/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "bindaudio.h"
#include "bindinterface.h"
#include "bindutil.h"

#include "beattracker.h"
#include "buffer.h"
#include "clip.h"
#include "mixer.h"
#include "audioport.h"
#include "onsetdetector.h"
#include "audioengine.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

using namespace audio;

namespace binding {

// object references to types in this package
HSQOBJECT AudioBeatTrackerObject;
HSQOBJECT AudioBufferObject;
HSQOBJECT AudioBufferEventObject;
HSQOBJECT AudioClipObject;
HSQOBJECT AudioConnectionObject;
HSQOBJECT AudioConnectorObject;
HSQOBJECT AudioMixerObject;
HSQOBJECT AudioMixerGainObject;
HSQOBJECT AudioMultiInputObject;
HSQOBJECT AudioMultiOutputObject;
HSQOBJECT AudioOnsetDetectorObject;
HSQOBJECT AudioOutputObject;
HSQOBJECT AudioInputObject;
HSQOBJECT AudioStereoInputObject;
HSQOBJECT AudioStereoOutputObject;

//
// Audio.BeatTracker class
//
#define BIND_CLASS_NAME "BeatTracker"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioBeatTrackerCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BeatTracker *obj;
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "connection" as Audio.Input
    BIND_GET_CHECK_OBJECT(connection, 1, Audio, Input, audio::AudioInputPort)
    // get parameter 2 "bpm" as float
    BIND_GET_CHECK_FLOAT(bpm, 2)
    // call the implementation
    BIND_WRAP_TRY(obj = BeatTrackerFactory::instance().newBeatTracker(*connection, bpm))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioBeatTrackerNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.BeatTracker");
}
//
// Audio.BeatTracker start
//
#define BIND_METHOD_NAME "start"
SQInteger AudioBeatTrackerstart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BeatTracker)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->start())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.BeatTracker stop
//
#define BIND_METHOD_NAME "stop"
SQInteger AudioBeatTrackerstop(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BeatTracker)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->stop())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioBeatTrackerToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.BeatTracker)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Buffer class
//
#define BIND_CLASS_NAME "Buffer"
SQInteger AudioBufferCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Audio.Buffer");
}

SQInteger AudioBufferNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Buffer");
}
SQInteger AudioBufferToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Buffer)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.BufferEvent class
//
#define BIND_CLASS_NAME "BufferEvent"
SQInteger AudioBufferEventCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Audio.BufferEvent");
}

SQInteger AudioBufferEventNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.BufferEvent");
}
SQInteger AudioBufferEventToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.BufferEvent)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Clip class
//
#define BIND_CLASS_NAME "Clip"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioClipCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Clip *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "file" as string
        BIND_GET_STRING(file, 1)
        // call the implementation
        BIND_WRAP_TRY(obj = ClipFactory::instance().loadClip(file))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &bufferObject) {
        // get parameter 1 "bytes" as buffer
        BIND_GET_OBJECT(bytes, 1, , buffer, bipscript::Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj = ClipFactory::instance().loadClip(*bytes))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, buffer}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, buffer}");
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioClipNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Clip");
}
//
// Audio.Clip buffer
//
#define BIND_METHOD_NAME "buffer"
SQInteger AudioClipbuffer(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Clip)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "index" as integer
    BIND_GET_CHECK_INTEGER(index, 1)
    // return value
    audio::Buffer* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->buffer(index))
    // push return value
    sq_pushobject(vm, AudioBufferObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret->addref());
    sq_setreleasehook(vm, -1, BindUtil::Decrement<audio::Buffer>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Audio.Clip lufs
//
#define BIND_METHOD_NAME "lufs"
SQInteger AudioCliplufs(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Clip)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQFloat ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->lufs())
    // push return value
    BIND_PUSH_FLOAT(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Audio.Clip regain
//
#define BIND_METHOD_NAME "regain"
SQInteger AudioClipregain(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Clip)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "factor" as float
    BIND_GET_CHECK_FLOAT(factor, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->regain(factor))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioClipToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Clip)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Connection class
//
#define BIND_CLASS_NAME "Connection"
SQInteger AudioConnectionCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Audio.Connection");
}

SQInteger AudioConnectionNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Connection");
}
SQInteger AudioConnectionToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Connection)", 18);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Connector class
//
#define BIND_CLASS_NAME "Connector"
SQInteger AudioConnectorCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Audio.Connector");
}

SQInteger AudioConnectorNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Connector");
}
//
// Audio.Connector play
//
#define BIND_METHOD_NAME "play"
SQInteger AudioConnectorplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "clip" as Audio.Buffer
    BIND_GET_CHECK_OBJECT(clip, 1, Audio, Buffer, audio::Buffer)
    // call the implementation
    BIND_WRAP_TRY(obj->play(*clip))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Connector remove
//
#define BIND_METHOD_NAME "remove"
SQInteger AudioConnectorremove(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "event" as Audio.BufferEvent
    BIND_GET_CHECK_OBJECT(event, 1, Audio, BufferEvent, audio::BufferEvent)
    // call the implementation
    BIND_WRAP_TRY(obj->remove(*event))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Connector schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger AudioConnectorschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "buffer" as Audio.Buffer
    BIND_GET_CHECK_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
    // get parameter 2 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 2)
    // return value
    audio::BufferEvent* ret;
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(ret = obj->schedule(*buffer, measure))
    } else {
        // get parameter 3 "clock" as Time.Clock
        BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
        // call the implementation
        BIND_WRAP_TRY(ret = obj->schedule(*buffer, measure, *clock))
    }
    // push return value
    sq_pushobject(vm, AudioBufferEventObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret->addref());
    sq_setreleasehook(vm, -1, BindUtil::Decrement<audio::BufferEvent>);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioConnectorToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Connector)", 17);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Mixer class
//
#define BIND_CLASS_NAME "Mixer"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioMixerCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Mixer *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "inputs" as integer
    BIND_GET_CHECK_INTEGER(inputs, 1)
    // get parameter 2 "outputs" as integer
    BIND_GET_CHECK_INTEGER(outputs, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = MixerFactory::instance().newMixer(inputs, outputs))
    } else {
        // get parameter 3 "gains" as array
        BIND_GET_CHECK_ARRAY(gains, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = MixerFactory::instance().newMixer(inputs, outputs, gains))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioMixerNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Mixer");
}
//
// Audio.Mixer property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger AudioMixerGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::Mixer)
    // dispatch appropriate getter
    if(!strcmp(key, "output")) {
        HSQOBJECT ret = obj->output();
        sq_pushobject(vm, ret);
        return 1;
    }
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger AudioMixerSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::Mixer)
    // dispatch appropriate setter
    if(!strcmp(key, "output")) {
        return sq_throwerror(vm, "property \"output\" is read-only");
    }
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer add
//
#define BIND_METHOD_NAME "add"
SQInteger AudioMixeradd(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioConnectionObject) {
        // get parameter 1 "connection" as Audio.Connection
        BIND_GET_OBJECT(connection, 1, Audio, Connection, audio::Connection)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*connection))
        } else {
            HSQOBJECT arg2;
            sq_getstackobj(vm, 3, &arg2);
            if(BIND_CHECK_FLOAT(2)) {
                // get parameter 2 "gain" as float
                BIND_GET_FLOAT(gain, 2)
                // call the implementation
                BIND_WRAP_TRY(obj->add(*connection, gain))
            }
            else if(arg2._type == OT_ARRAY) {
                BIND_NARGS_CHECK_MIN(2) // check min parameter count
                // get parameter 2 "gain" as array
                BIND_GET_ARRAY(gain, 2)
                // call the implementation
                BIND_WRAP_TRY(obj->add(*connection, gain))
            }
            else {
                return sq_throwerror(vm, "argument 2 is not of type {float, array}");
            }
        }
    }
    else if(audio::Source *source = getAudioSource(arg1type, &arg1)) {
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*source))
        } else {
            HSQOBJECT arg2;
            sq_getstackobj(vm, 3, &arg2);
            if(BIND_CHECK_FLOAT(2)) {
                // get parameter 2 "gain" as float
                BIND_GET_FLOAT(gain, 2)
                // call the implementation
                BIND_WRAP_TRY(obj->add(*source, gain))
            }
            else if(arg2._type == OT_ARRAY) {
                BIND_NARGS_CHECK_MIN(2) // check min parameter count
                // get parameter 2 "gains" as array
                BIND_GET_ARRAY(gains, 2)
                // call the implementation
                BIND_WRAP_TRY(obj->add(*source, gains))
            }
            else {
                return sq_throwerror(vm, "argument 2 is not of type {float, array}");
            }
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Connection, Audio.Source}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Connection, Audio.Source}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer gain
//
#define BIND_METHOD_NAME "gain"
SQInteger AudioMixergain(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    // get parameter 1 "input" as integer
    BIND_GET_CHECK_INTEGER(input, 1)
    HSQOBJECT arg2;
    sq_getstackobj(vm, 3, &arg2);
    if(arg2._type == OT_INTEGER) {
        BIND_NARGS_CHECK_MIN(3) // check min parameter count
        BIND_NARGS_CHECK_MAX(3) // check max parameter count
        // get parameter 2 "output" as integer
        BIND_GET_INTEGER(output, 2)
        // get parameter 3 "gain" as float
        BIND_GET_CHECK_FLOAT(gain, 3)
        // call the implementation
        BIND_WRAP_TRY(obj->gain(input, output, gain))
    }
    else if(arg2._type == OT_ARRAY) {
        BIND_NARGS_CHECK_MAX(2) // check max parameter count
        // get parameter 2 "gains" as array
        BIND_GET_ARRAY(gains, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->gain(input, gains))
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {integer, array}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer grab
//
#define BIND_METHOD_NAME "grab"
SQInteger AudioMixergrab(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "length" as float
    BIND_GET_CHECK_FLOAT(length, 1)
    // get parameter 2 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 2)
    // get parameter 3 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 3)
    // call the implementation
    BIND_WRAP_TRY(obj->grab(length, handler, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer play
//
#define BIND_METHOD_NAME "play"
SQInteger AudioMixerplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Mixer schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger AudioMixerschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Mixer)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioMixerGainObject) {
        // get parameter 1 "gain" as Audio.MixerGain
        BIND_GET_OBJECT(gain, 1, Audio, MixerGain, audio::MixerGain)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*gain, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*gain, measure, *clock))
        }
    }
    else if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->audio::Sink::schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.MixerGain, Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.MixerGain, Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioMixerToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Mixer)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.MixerGain class
//
#define BIND_CLASS_NAME "MixerGain"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioMixerGainCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MixerGain *obj;
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "input" as integer
    BIND_GET_CHECK_INTEGER(input, 1)
    // get parameter 2 "output" as integer
    BIND_GET_CHECK_INTEGER(output, 2)
    // get parameter 3 "amount" as float
    BIND_GET_CHECK_FLOAT(amount, 3)
    // call the implementation
    BIND_WRAP_TRY(obj = new MixerGain(input, output, amount))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<MixerGain>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Audio.MixerGain property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger AudioMixerGainGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MixerGain)
    // dispatch appropriate getter
    if(!strcmp(key, "amount")) {
        SQFloat ret = obj->amount();
        BIND_PUSH_FLOAT(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MixerGain property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger AudioMixerGainSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MixerGain)
    // dispatch appropriate setter
    if(!strcmp(key, "amount")) {
        return sq_throwerror(vm, "property \"amount\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger AudioMixerGainToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.MixerGain)", 17);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.MultiInput class
//
#define BIND_CLASS_NAME "MultiInput"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioMultiInputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MultiInput *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    // get parameter 2 "channels" as integer
    BIND_GET_CHECK_INTEGER(channels, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = InputPortSetFactory::instance().newInputPortSet(name, channels))
    } else {
        // get parameter 3 "autoconnect" as bool
        BIND_GET_CHECK_BOOL(autoconnect, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = InputPortSetFactory::instance().newInputPortSet(name, channels, autoconnect))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioMultiInputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.MultiInput");
}
//
// Audio.MultiInput property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger AudioMultiInputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MultiInput)
    // dispatch appropriate getter
    if(!strcmp(key, "output")) {
        HSQOBJECT ret = obj->output();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MultiInput property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger AudioMultiInputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MultiInput)
    // dispatch appropriate setter
    if(!strcmp(key, "output")) {
        return sq_throwerror(vm, "property \"output\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MultiInput connect
//
#define BIND_METHOD_NAME "connect"
SQInteger AudioMultiInputconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MultiInput)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "index" as integer
    BIND_GET_CHECK_INTEGER(index, 1)
    HSQOBJECT arg2;
    sq_getstackobj(vm, 3, &arg2);
    if(arg2._type == OT_STRING) {
        // get parameter 2 "name" as string
        BIND_GET_STRING(name, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(index, name))
    }
    else if(arg2._type == OT_INSTANCE) {
    SQUserPointer arg2type = bip_getinstancetypetag(&arg2);
    if(arg2type == &regexObject) {
        // get parameter 2 "name" as regex
        BIND_GET_OBJECT(name, 2, , regex, bipscript::Regex)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(index, *name))
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.MultiInput grab
//
#define BIND_METHOD_NAME "grab"
SQInteger AudioMultiInputgrab(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MultiInput)
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "length" as float
    BIND_GET_CHECK_FLOAT(length, 1)
    // get parameter 2 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 2)
    // get parameter 3 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 3)
    // call the implementation
    BIND_WRAP_TRY(obj->grab(length, handler, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioMultiInputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.MultiInput)", 18);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.MultiOutput class
//
#define BIND_CLASS_NAME "MultiOutput"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioMultiOutputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MultiOutput *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    // get parameter 2 "channels" as integer
    BIND_GET_CHECK_INTEGER(channels, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newOutputPortSet(name, channels))
    } else {
        // get parameter 3 "autoconnect" as bool
        BIND_GET_CHECK_BOOL(autoconnect, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newOutputPortSet(name, channels, autoconnect))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioMultiOutputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.MultiOutput");
}
//
// Audio.MultiOutput property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger AudioMultiOutputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MultiOutput)
    // dispatch appropriate getter
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MultiOutput property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger AudioMultiOutputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::MultiOutput)
    // dispatch appropriate setter
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.MultiOutput play
//
#define BIND_METHOD_NAME "play"
SQInteger AudioMultiOutputplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MultiOutput)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.MultiOutput schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger AudioMultiOutputschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MultiOutput)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioMultiOutputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.MultiOutput)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.OnsetDetector class
//
#define BIND_CLASS_NAME "OnsetDetector"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioOnsetDetectorCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    OnsetDetector *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = OnsetDetectorFactory::instance().newOnsetDetector())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioOnsetDetectorNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.OnsetDetector");
}
//
// Audio.OnsetDetector property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger AudioOnsetDetectorGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::OnsetDetector)
    // dispatch appropriate getter
    if(!strcmp(key, "threshold")) {
        SQFloat ret = obj->threshold();
        BIND_PUSH_FLOAT(ret);
        return 1;
    }
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.OnsetDetector property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger AudioOnsetDetectorSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::OnsetDetector)
    // dispatch appropriate setter
    if(!strcmp(key, "threshold")) {
        // get parameter 2 "threshold" as float
        BIND_GET_CHECK_FLOAT(threshold, 2)
        BIND_WRAP_TRY(obj->threshold(threshold))
        return 0;
    }
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.OnsetDetector onOnset
//
#define BIND_METHOD_NAME "onOnset"
SQInteger AudioOnsetDetectoronOnset(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(OnsetDetector)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onOnset(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.OnsetDetector play
//
#define BIND_METHOD_NAME "play"
SQInteger AudioOnsetDetectorplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(OnsetDetector)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.OnsetDetector schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger AudioOnsetDetectorschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(OnsetDetector)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioOnsetDetectorToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.OnsetDetector)", 21);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Output class
//
#define BIND_CLASS_NAME "Output"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioOutputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Output *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newAudioOutputPort(name))
    } else {
        HSQOBJECT arg2;
        sq_getstackobj(vm, 3, &arg2);
        if(arg2._type == OT_STRING) {
            // get parameter 2 "connection" as string
            BIND_GET_STRING(connection, 2)
            // call the implementation
            BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newAudioOutputPort(name, connection))
        }
        else if(arg2._type == OT_BOOL) {
            BIND_NARGS_CHECK_MIN(2) // check min parameter count
            // get parameter 2 "autoconnect" as bool
            BIND_GET_BOOL(autoconnect, 2)
            // call the implementation
            BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newAudioOutputPort(name, autoconnect))
        }
        else {
            return sq_throwerror(vm, "argument 2 is not of type {string, bool}");
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioOutputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Output");
}
//
// Audio.Output property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger AudioOutputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::Output)
    // dispatch appropriate getter
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Output property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger AudioOutputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::Output)
    // dispatch appropriate setter
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Output play
//
#define BIND_METHOD_NAME "play"
SQInteger AudioOutputplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Output)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.Output schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger AudioOutputschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Output)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioOutputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Output)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.Input class
//
#define BIND_CLASS_NAME "Input"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioInputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    AudioInputPort *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = AudioInputPortFactory::instance().newAudioInputPort(name))
    } else {
        // get parameter 2 "connection" as string
        BIND_GET_CHECK_STRING(connection, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = AudioInputPortFactory::instance().newAudioInputPort(name, connection))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioInputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.Input");
}
//
// Audio.Input property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger AudioInputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::AudioInputPort)
    // dispatch appropriate getter
    if(!strcmp(key, "output")) {
        HSQOBJECT ret = obj->output();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Input property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger AudioInputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::AudioInputPort)
    // dispatch appropriate setter
    if(!strcmp(key, "output")) {
        return sq_throwerror(vm, "property \"output\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.Input grab
//
#define BIND_METHOD_NAME "grab"
SQInteger AudioInputgrab(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(AudioInputPort)
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "length" as float
    BIND_GET_CHECK_FLOAT(length, 1)
    // get parameter 2 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 2)
    // get parameter 3 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 3)
    // call the implementation
    BIND_WRAP_TRY(obj->grab(length, handler, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioInputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.Input)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.StereoInput class
//
#define BIND_CLASS_NAME "StereoInput"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioStereoInputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    StereoInput *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = InputPortSetFactory::instance().newStereoInputPort(name))
    } else {
        // get parameter 2 "autoconnect" as bool
        BIND_GET_CHECK_BOOL(autoconnect, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = InputPortSetFactory::instance().newStereoInputPort(name, autoconnect))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioStereoInputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.StereoInput");
}
//
// Audio.StereoInput property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger AudioStereoInputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::StereoInput)
    // dispatch appropriate getter
    if(!strcmp(key, "output")) {
        HSQOBJECT ret = obj->output();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.StereoInput property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger AudioStereoInputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::StereoInput)
    // dispatch appropriate setter
    if(!strcmp(key, "output")) {
        return sq_throwerror(vm, "property \"output\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.StereoInput connect
//
#define BIND_METHOD_NAME "connect"
SQInteger AudioStereoInputconnect(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(StereoInput)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "index" as integer
    BIND_GET_CHECK_INTEGER(index, 1)
    HSQOBJECT arg2;
    sq_getstackobj(vm, 3, &arg2);
    if(arg2._type == OT_STRING) {
        // get parameter 2 "name" as string
        BIND_GET_STRING(name, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(index, name))
    }
    else if(arg2._type == OT_INSTANCE) {
    SQUserPointer arg2type = bip_getinstancetypetag(&arg2);
    if(arg2type == &regexObject) {
        // get parameter 2 "name" as regex
        BIND_GET_OBJECT(name, 2, , regex, bipscript::Regex)
        // call the implementation
        BIND_WRAP_TRY(obj->connect(index, *name))
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.StereoInput grab
//
#define BIND_METHOD_NAME "grab"
SQInteger AudioStereoInputgrab(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(StereoInput)
    BIND_NARGS_CHECK_EXACT(3) // check exact parameter count
    // get parameter 1 "length" as float
    BIND_GET_CHECK_FLOAT(length, 1)
    // get parameter 2 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 2)
    // get parameter 3 "measure" as float
    BIND_GET_CHECK_FLOAT(measure, 3)
    // call the implementation
    BIND_WRAP_TRY(obj->grab(length, handler, measure))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioStereoInputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.StereoInput)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Audio.StereoOutput class
//
#define BIND_CLASS_NAME "StereoOutput"
#define BIND_METHOD_NAME "constructor"
SQInteger AudioStereoOutputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    StereoOutput *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newStereoOutputPort(name))
    } else {
        // get parameter 2 "autoconnect" as bool
        BIND_GET_CHECK_BOOL(autoconnect, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = OutputPortSetFactory::instance().newStereoOutputPort(name, autoconnect))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger AudioStereoOutputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Audio.StereoOutput");
}
//
// Audio.StereoOutput property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger AudioStereoOutputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::StereoOutput)
    // dispatch appropriate getter
    if(!strcmp(key, "input")) {
        HSQOBJECT ret = obj->input();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.StereoOutput property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger AudioStereoOutputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(audio::StereoOutput)
    // dispatch appropriate setter
    if(!strcmp(key, "input")) {
        return sq_throwerror(vm, "property \"input\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Audio.StereoOutput play
//
#define BIND_METHOD_NAME "play"
SQInteger AudioStereoOutputplay(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(StereoOutput)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*buffer))
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // call the implementation
        BIND_WRAP_TRY(obj->play(*clip))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Audio.StereoOutput schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger AudioStereoOutputschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(StereoOutput)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &AudioBufferObject) {
        // get parameter 1 "buffer" as Audio.Buffer
        BIND_GET_OBJECT(buffer, 1, Audio, Buffer, audio::Buffer)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &AudioClipObject) {
        // get parameter 1 "clip" as Audio.Clip
        BIND_GET_OBJECT(clip, 1, Audio, Clip, audio::Clip)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*clip, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Audio.Buffer, Audio.Clip}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger AudioStereoOutputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Audio.StereoOutput)", 20);
    return 1;
}
#undef BIND_CLASS_NAME


void bindAudio(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("Audio", 5);

    // create class Audio.BeatTracker
    binding.startClass("BeatTracker", 11, AudioBeatTrackerObject);
    binding.addMethod("constructor", 11, &AudioBeatTrackerCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioBeatTrackerNoclone); // unclonable
    binding.addMethod("start", 5, &AudioBeatTrackerstart);
    binding.addMethod("stop", 4, &AudioBeatTrackerstop);
    binding.addMethod("_tostring", 9, &AudioBeatTrackerToString);
    binding.endClass();

    // create class Audio.Buffer
    binding.startClass("Buffer", 6, AudioBufferObject);
    binding.addMethod("constructor", 11, &AudioBufferCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioBufferNoclone); // unclonable
    binding.addMethod("_tostring", 9, &AudioBufferToString);
    binding.endClass();

    // create class Audio.BufferEvent
    binding.startClass("BufferEvent", 11, AudioBufferEventObject);
    binding.addMethod("constructor", 11, &AudioBufferEventCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioBufferEventNoclone); // unclonable
    binding.addMethod("_tostring", 9, &AudioBufferEventToString);
    binding.endClass();

    // create class Audio.Clip
    binding.startClass("Clip", 4, AudioClipObject);
    binding.addMethod("constructor", 11, &AudioClipCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioClipNoclone); // unclonable
    binding.addMethod("buffer", 6, &AudioClipbuffer);
    binding.addMethod("lufs", 4, &AudioCliplufs);
    binding.addMethod("regain", 6, &AudioClipregain);
    binding.addMethod("_tostring", 9, &AudioClipToString);
    binding.endClass();

    // create class Audio.Connection
    binding.startClass("Connection", 10, AudioConnectionObject);
    binding.addMethod("constructor", 11, &AudioConnectionCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioConnectionNoclone); // unclonable
    binding.addMethod("_tostring", 9, &AudioConnectionToString);
    binding.endClass();

    // create class Audio.Connector
    binding.startClass("Connector", 9, AudioConnectorObject);
    binding.addMethod("constructor", 11, &AudioConnectorCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioConnectorNoclone); // unclonable
    binding.addMethod("play", 4, &AudioConnectorplay);
    binding.addMethod("remove", 6, &AudioConnectorremove);
    binding.addMethod("schedule", 8, &AudioConnectorschedule);
    binding.addMethod("_tostring", 9, &AudioConnectorToString);
    binding.endClass();

    // create class Audio.Mixer
    binding.startClass("Mixer", 5, AudioMixerObject);
    binding.addMethod("constructor", 11, &AudioMixerCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioMixerNoclone); // unclonable
    binding.addMethod("_get", 4, &AudioMixerGetters);
    binding.addMethod("_set", 4, &AudioMixerSetters);
    binding.addMethod("add", 3, &AudioMixeradd);
    binding.addMethod("gain", 4, &AudioMixergain);
    binding.addMethod("grab", 4, &AudioMixergrab);
    binding.addMethod("play", 4, &AudioMixerplay);
    binding.addMethod("schedule", 8, &AudioMixerschedule);
    binding.addMethod("_tostring", 9, &AudioMixerToString);
    binding.endClass();

    // create class Audio.MixerGain
    binding.startClass("MixerGain", 9, AudioMixerGainObject);
    binding.addMethod("constructor", 11, &AudioMixerGainCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<MixerGain>); // clone
    binding.addMethod("_get", 4, &AudioMixerGainGetters);
    binding.addMethod("_set", 4, &AudioMixerGainSetters);
    binding.addMethod("_tostring", 9, &AudioMixerGainToString);
    binding.endClass();

    // create class Audio.MultiInput
    binding.startClass("MultiInput", 10, AudioMultiInputObject);
    binding.addMethod("constructor", 11, &AudioMultiInputCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioMultiInputNoclone); // unclonable
    binding.addMethod("_get", 4, &AudioMultiInputGetters);
    binding.addMethod("_set", 4, &AudioMultiInputSetters);
    binding.addMethod("connect", 7, &AudioMultiInputconnect);
    binding.addMethod("grab", 4, &AudioMultiInputgrab);
    binding.addMethod("_tostring", 9, &AudioMultiInputToString);
    binding.endClass();

    // create class Audio.MultiOutput
    binding.startClass("MultiOutput", 11, AudioMultiOutputObject);
    binding.addMethod("constructor", 11, &AudioMultiOutputCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioMultiOutputNoclone); // unclonable
    binding.addMethod("_get", 4, &AudioMultiOutputGetters);
    binding.addMethod("_set", 4, &AudioMultiOutputSetters);
    binding.addMethod("play", 4, &AudioMultiOutputplay);
    binding.addMethod("schedule", 8, &AudioMultiOutputschedule);
    binding.addMethod("_tostring", 9, &AudioMultiOutputToString);
    binding.endClass();

    // create class Audio.OnsetDetector
    binding.startClass("OnsetDetector", 13, AudioOnsetDetectorObject);
    binding.addMethod("constructor", 11, &AudioOnsetDetectorCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioOnsetDetectorNoclone); // unclonable
    binding.addMethod("_get", 4, &AudioOnsetDetectorGetters);
    binding.addMethod("_set", 4, &AudioOnsetDetectorSetters);
    binding.addMethod("onOnset", 7, &AudioOnsetDetectoronOnset);
    binding.addMethod("play", 4, &AudioOnsetDetectorplay);
    binding.addMethod("schedule", 8, &AudioOnsetDetectorschedule);
    binding.addMethod("_tostring", 9, &AudioOnsetDetectorToString);
    binding.endClass();

    // create class Audio.Output
    binding.startClass("Output", 6, AudioOutputObject);
    binding.addMethod("constructor", 11, &AudioOutputCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioOutputNoclone); // unclonable
    binding.addMethod("_get", 4, &AudioOutputGetters);
    binding.addMethod("_set", 4, &AudioOutputSetters);
    binding.addMethod("play", 4, &AudioOutputplay);
    binding.addMethod("schedule", 8, &AudioOutputschedule);
    binding.addMethod("_tostring", 9, &AudioOutputToString);
    binding.endClass();

    // create class Audio.Input
    binding.startClass("Input", 5, AudioInputObject);
    binding.addMethod("constructor", 11, &AudioInputCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioInputNoclone); // unclonable
    binding.addMethod("_get", 4, &AudioInputGetters);
    binding.addMethod("_set", 4, &AudioInputSetters);
    binding.addMethod("grab", 4, &AudioInputgrab);
    binding.addMethod("_tostring", 9, &AudioInputToString);
    binding.endClass();

    // create class Audio.StereoInput
    binding.startClass("StereoInput", 11, AudioStereoInputObject);
    binding.addMethod("constructor", 11, &AudioStereoInputCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioStereoInputNoclone); // unclonable
    binding.addMethod("_get", 4, &AudioStereoInputGetters);
    binding.addMethod("_set", 4, &AudioStereoInputSetters);
    binding.addMethod("connect", 7, &AudioStereoInputconnect);
    binding.addMethod("grab", 4, &AudioStereoInputgrab);
    binding.addMethod("_tostring", 9, &AudioStereoInputToString);
    binding.endClass();

    // create class Audio.StereoOutput
    binding.startClass("StereoOutput", 12, AudioStereoOutputObject);
    binding.addMethod("constructor", 11, &AudioStereoOutputCtor); // ctor
    binding.addMethod("_cloned", 7, &AudioStereoOutputNoclone); // unclonable
    binding.addMethod("_get", 4, &AudioStereoOutputGetters);
    binding.addMethod("_set", 4, &AudioStereoOutputSetters);
    binding.addMethod("play", 4, &AudioStereoOutputplay);
    binding.addMethod("schedule", 8, &AudioStereoOutputschedule);
    binding.addMethod("_tostring", 9, &AudioStereoOutputToString);
    binding.endClass();

    binding.endTable(); // end package table
}
}}
