/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef BINDINTERFACE_H
#define BINDINTERFACE_H

#include "bipsquirrel.h"

namespace bipscript {

namespace audio { class Source; }
namespace audio { class Sink; }
namespace io { class Path; }
namespace midi { class Message; }
namespace midi { class Source; }
namespace midi { class Sink; }
namespace time { class Clock; }

namespace binding
{
    // argument type conversion methods
    audio::Source *getAudioSource(HSQUIRRELVM &vm, int index);
    audio::Source *getAudioSource(SQUserPointer type, HSQOBJECT *arg);
    audio::Sink *getAudioSink(HSQUIRRELVM &vm, int index);
    audio::Sink *getAudioSink(SQUserPointer type, HSQOBJECT *arg);
    io::Path *getIOPath(HSQUIRRELVM &vm, int index);
    io::Path *getIOPath(SQUserPointer type, HSQOBJECT *arg);
    midi::Message *getMidiMessage(HSQUIRRELVM &vm, int index);
    midi::Message *getMidiMessage(SQUserPointer type, HSQOBJECT *arg);
    midi::Source *getMidiSource(HSQUIRRELVM &vm, int index);
    midi::Source *getMidiSource(SQUserPointer type, HSQOBJECT *arg);
    midi::Sink *getMidiSink(HSQUIRRELVM &vm, int index);
    midi::Sink *getMidiSink(SQUserPointer type, HSQOBJECT *arg);
    time::Clock *getTimeClock(HSQUIRRELVM &vm, int index);
    time::Clock *getTimeClock(SQUserPointer type, HSQOBJECT *arg);

    // return type conversion methods 
    void pushIOPath(HSQUIRRELVM &vm, io::Path *obj);
    void pushMidiMessage(HSQUIRRELVM &vm, midi::Message *obj);
    void pushTimeClock(HSQUIRRELVM &vm, time::Clock *obj);

    // resolve methods - implemented elsewhere
    audio::Source *resolve_audiosource(SQUserPointer obj);
    audio::Sink *resolve_audiosink(SQUserPointer obj);
}
}

#endif // BINDINTERFACE_H
