/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "bindio.h"
#include "bindinterface.h"
#include "bindutil.h"

#include "io.h"
#include "jsontype.h"
#include "xmltype.h"
#include "zipfile.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

using namespace io;

namespace binding {

// object references to types in this package
HSQOBJECT IOFileObject;
HSQOBJECT IOFolderObject;
HSQOBJECT IOJsonObject;
HSQOBJECT IOXmlObject;
HSQOBJECT IOZipFileObject;
HSQOBJECT IOZipFolderObject;
HSQOBJECT IOZipObject;

//
// IO.File class
//
#define BIND_CLASS_NAME "File"
#define BIND_METHOD_NAME "constructor"
SQInteger IOFileCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    File *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "path" as string
    BIND_GET_CHECK_STRING(path, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new File(path))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<File>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.File property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger IOFileGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::File)
    // dispatch appropriate getter
    if(!strcmp(key, "path")) {
        std::string ret = obj->path();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    if(!strcmp(key, "name")) {
        std::string ret = obj->name();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.File property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger IOFileSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::File)
    // dispatch appropriate setter
    if(!strcmp(key, "path")) {
        return sq_throwerror(vm, "property \"path\" is read-only");
    }
    if(!strcmp(key, "name")) {
        return sq_throwerror(vm, "property \"name\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.File bytes
//
#define BIND_METHOD_NAME "bytes"
SQInteger IOFilebytes(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    bipscript::Buffer* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->bytes())
    // push return value
    sq_pushobject(vm, bufferObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret->addref());
    sq_setreleasehook(vm, -1, BindUtil::Decrement<bipscript::Buffer>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.File exists
//
#define BIND_METHOD_NAME "exists"
SQInteger IOFileexists(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQBool ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->exists())
    // push return value
    BIND_PUSH_BOOL(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.File folder
//
#define BIND_METHOD_NAME "folder"
SQInteger IOFilefolder(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQBool ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->folder())
    // push return value
    BIND_PUSH_BOOL(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.File len
//
#define BIND_METHOD_NAME "len"
SQInteger IOFilelen(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->len())
    // push return value
    BIND_PUSH_INTEGER(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.File readAll
//
#define BIND_METHOD_NAME "readAll"
SQInteger IOFilereadAll(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    std::string ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->readAll())
    // push return value
    BIND_PUSH_STRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.File reader
//
#define BIND_METHOD_NAME "reader"
SQInteger IOFilereader(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    bipscript::Reader* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->reader())
    // push return value
    sq_pushobject(vm, readerObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret->addref());
    sq_setreleasehook(vm, -1, BindUtil::Decrement<bipscript::Reader>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.File remove
//
#define BIND_METHOD_NAME "remove"
SQInteger IOFileremove(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQBool ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->remove())
    // push return value
    BIND_PUSH_BOOL(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.File text
//
#define BIND_METHOD_NAME "text"
SQInteger IOFiletext(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    const SQChar* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->text())
    // push return value
    BIND_PUSH_CSTRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.File writeAll
//
#define BIND_METHOD_NAME "writeAll"
SQInteger IOFilewriteAll(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "body" as string
    BIND_GET_CHECK_STRING(body, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->writeAll(body))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// IO.File writer
//
#define BIND_METHOD_NAME "writer"
SQInteger IOFilewriter(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    bipscript::Writer* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->writer())
    // push return value
    sq_pushobject(vm, writerObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<bipscript::Writer>);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger IOFileToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(IO.File)", 9);
    return 1;
}
#undef BIND_CLASS_NAME

//
// IO.Folder class
//
#define BIND_CLASS_NAME "Folder"
#define BIND_METHOD_NAME "constructor"
SQInteger IOFolderCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Folder *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "path" as string
    BIND_GET_CHECK_STRING(path, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Folder(path))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Folder>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.Folder property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger IOFolderGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::Folder)
    // dispatch appropriate getter
    if(!strcmp(key, "path")) {
        std::string ret = obj->path();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    if(!strcmp(key, "name")) {
        std::string ret = obj->name();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.Folder property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger IOFolderSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::Folder)
    // dispatch appropriate setter
    if(!strcmp(key, "path")) {
        return sq_throwerror(vm, "property \"path\" is read-only");
    }
    if(!strcmp(key, "name")) {
        return sq_throwerror(vm, "property \"name\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.Folder children
//
#define BIND_METHOD_NAME "children"
SQInteger IOFolderchildren(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Folder)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    bipscript::Iterator* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->children())
    // push return value
    sq_pushobject(vm, iteratorObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<bipscript::Iterator>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.Folder exists
//
#define BIND_METHOD_NAME "exists"
SQInteger IOFolderexists(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Folder)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQBool ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->exists())
    // push return value
    BIND_PUSH_BOOL(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.Folder folder
//
#define BIND_METHOD_NAME "folder"
SQInteger IOFolderfolder(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Folder)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQBool ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->folder())
    // push return value
    BIND_PUSH_BOOL(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.Folder mkdir
//
#define BIND_METHOD_NAME "mkdir"
SQInteger IOFoldermkdir(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Folder)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->mkdir())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger IOFolderToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(IO.Folder)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

//
// IO.Json class
//
#define BIND_CLASS_NAME "Json"
#define BIND_METHOD_NAME "constructor"
SQInteger IOJsonCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Json *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Json())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Json>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.Json property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger IOJsonGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::Json)
    // dispatch appropriate getter
    if(!strcmp(key, "verbose")) {
        SQBool ret = obj->verbose();
        BIND_PUSH_BOOL(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.Json property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger IOJsonSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::Json)
    // dispatch appropriate setter
    if(!strcmp(key, "verbose")) {
        // get parameter 2 "verbose" as bool
        BIND_GET_CHECK_BOOL(verbose, 2)
        BIND_WRAP_TRY(obj->verbose(verbose))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.Json read
//
#define BIND_METHOD_NAME "read"
SQInteger IOJsonread(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Json)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "json" as string
        BIND_GET_STRING(json, 1)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->read(vm, json))
        } else {
            // get parameter 2 "clazz" as class
            BIND_GET_CHECK_CLASS(clazz, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->read(vm, json, clazz))
        }
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &IOFileObject) {
        // get parameter 1 "file" as IO.File
        BIND_GET_OBJECT(file, 1, IO, File, io::File)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->read(vm, *file))
        } else {
            // get parameter 2 "clazz" as class
            BIND_GET_CHECK_CLASS(clazz, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->read(vm, *file, clazz))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, IO.File}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, IO.File}");
    }
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.Json write
//
#define BIND_METHOD_NAME "write"
SQInteger IOJsonwrite(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Json)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    // get parameter 1 "object" as any
    BIND_GET_CHECK_ANY(object, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj->write(object))
    } else {
        HSQOBJECT arg2;
        sq_getstackobj(vm, 3, &arg2);
        if(arg2._type == OT_BOOL) {
            BIND_NARGS_CHECK_MAX(2) // check max parameter count
            // get parameter 2 "pretty" as bool
            BIND_GET_BOOL(pretty, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->write(object, pretty))
        }
        else if(arg2._type == OT_INSTANCE) {
        SQUserPointer arg2type = bip_getinstancetypetag(&arg2);
        if(arg2type == &IOFileObject) {
            BIND_NARGS_CHECK_MIN(2) // check min parameter count
            BIND_NARGS_CHECK_MAX(3) // check max parameter count
            // get parameter 2 "file" as IO.File
            BIND_GET_OBJECT(file, 2, IO, File, io::File)
            if(numargs == 3) {
                // call the implementation
                BIND_WRAP_TRY(obj->write(object, *file))
            } else {
                // get parameter 3 "pretty" as bool
                BIND_GET_CHECK_BOOL(pretty, 3)
                // call the implementation
                BIND_WRAP_TRY(obj->write(object, *file, pretty))
            }
        }
        else {
            return sq_throwerror(vm, "argument 2 is not of type {bool, IO.File}");
        }
        }
        else {
            return sq_throwerror(vm, "argument 2 is not of type {bool, IO.File}");
        }
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger IOJsonToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(IO.Json)", 9);
    return 1;
}
#undef BIND_CLASS_NAME

//
// IO.Xml class
//
#define BIND_CLASS_NAME "Xml"
#define BIND_METHOD_NAME "constructor"
SQInteger IOXmlCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Xml *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Xml())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Xml>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.Xml property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger IOXmlGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::Xml)
    // dispatch appropriate getter
    if(!strcmp(key, "textfield")) {
        std::string ret = obj->textfield();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    if(!strcmp(key, "verbose")) {
        SQBool ret = obj->verbose();
        BIND_PUSH_BOOL(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.Xml property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger IOXmlSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::Xml)
    // dispatch appropriate setter
    if(!strcmp(key, "textfield")) {
        // get parameter 2 "textfield" as string
        BIND_GET_CHECK_STRING(textfield, 2)
        BIND_WRAP_TRY(obj->textfield(textfield))
        return 0;
    }
    if(!strcmp(key, "verbose")) {
        // get parameter 2 "verbose" as bool
        BIND_GET_CHECK_BOOL(verbose, 2)
        BIND_WRAP_TRY(obj->verbose(verbose))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.Xml read
//
#define BIND_METHOD_NAME "read"
SQInteger IOXmlread(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Xml)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "json" as string
        BIND_GET_STRING(json, 1)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->read(vm, json))
        } else {
            // get parameter 2 "clazz" as class
            BIND_GET_CHECK_CLASS(clazz, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->read(vm, json, clazz))
        }
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &IOFileObject) {
        // get parameter 1 "file" as IO.File
        BIND_GET_OBJECT(file, 1, IO, File, io::File)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->read(vm, *file))
        } else {
            // get parameter 2 "clazz" as class
            BIND_GET_CHECK_CLASS(clazz, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->read(vm, *file, clazz))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, IO.File}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {string, IO.File}");
    }
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger IOXmlToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(IO.Xml)", 8);
    return 1;
}
#undef BIND_CLASS_NAME

//
// IO.ZipFile class
//
#define BIND_CLASS_NAME "ZipFile"
SQInteger IOZipFileCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate IO.ZipFile");
}

SQInteger IOZipFileNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type IO.ZipFile");
}
//
// IO.ZipFile property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger IOZipFileGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::ZipFile)
    // dispatch appropriate getter
    if(!strcmp(key, "path")) {
        std::string ret = obj->path();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    if(!strcmp(key, "name")) {
        std::string ret = obj->name();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.ZipFile property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger IOZipFileSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::ZipFile)
    // dispatch appropriate setter
    if(!strcmp(key, "path")) {
        return sq_throwerror(vm, "property \"path\" is read-only");
    }
    if(!strcmp(key, "name")) {
        return sq_throwerror(vm, "property \"name\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.ZipFile bytes
//
#define BIND_METHOD_NAME "bytes"
SQInteger IOZipFilebytes(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(ZipFile)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    bipscript::Buffer* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->bytes())
    // push return value
    sq_pushobject(vm, bufferObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret->addref());
    sq_setreleasehook(vm, -1, BindUtil::Decrement<bipscript::Buffer>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.ZipFile exists
//
#define BIND_METHOD_NAME "exists"
SQInteger IOZipFileexists(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(ZipFile)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQBool ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->exists())
    // push return value
    BIND_PUSH_BOOL(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.ZipFile folder
//
#define BIND_METHOD_NAME "folder"
SQInteger IOZipFilefolder(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(ZipFile)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQBool ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->folder())
    // push return value
    BIND_PUSH_BOOL(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.ZipFile size
//
#define BIND_METHOD_NAME "size"
SQInteger IOZipFilesize(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(ZipFile)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->size())
    // push return value
    BIND_PUSH_INTEGER(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.ZipFile text
//
#define BIND_METHOD_NAME "text"
SQInteger IOZipFiletext(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(ZipFile)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    const SQChar* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->text())
    // push return value
    BIND_PUSH_CSTRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger IOZipFileToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(IO.ZipFile)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// IO.ZipFolder class
//
#define BIND_CLASS_NAME "ZipFolder"
SQInteger IOZipFolderCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate IO.ZipFolder");
}

SQInteger IOZipFolderNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type IO.ZipFolder");
}
//
// IO.ZipFolder property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger IOZipFolderGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::ZipFolder)
    // dispatch appropriate getter
    if(!strcmp(key, "path")) {
        std::string ret = obj->path();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    if(!strcmp(key, "name")) {
        std::string ret = obj->name();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.ZipFolder property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger IOZipFolderSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(io::ZipFolder)
    // dispatch appropriate setter
    if(!strcmp(key, "path")) {
        return sq_throwerror(vm, "property \"path\" is read-only");
    }
    if(!strcmp(key, "name")) {
        return sq_throwerror(vm, "property \"name\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// IO.ZipFolder exists
//
#define BIND_METHOD_NAME "exists"
SQInteger IOZipFolderexists(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(ZipFolder)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQBool ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->exists())
    // push return value
    BIND_PUSH_BOOL(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.ZipFolder folder
//
#define BIND_METHOD_NAME "folder"
SQInteger IOZipFolderfolder(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(ZipFolder)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQBool ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->folder())
    // push return value
    BIND_PUSH_BOOL(ret);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger IOZipFolderToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(IO.ZipFolder)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// IO.Zip class
//
#define BIND_CLASS_NAME "Zip"
#define BIND_METHOD_NAME "constructor"
SQInteger IOZipCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Zip *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "path" as string
    BIND_GET_CHECK_STRING(path, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Zip(path))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Zip>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.Zip property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger IOZipGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        BIND_GET_THIS_PROPERTY(io::Zip)
        io::Path* ret = (*obj)[index];
        if(ret) {
            pushIOPath(vm, ret);
            return 1;
        }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME
//
// IO.Zip _nexti
//
SQInteger IOZipNexti(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
	BIND_GET_THIS_PROPERTY(Zip)
    if (argtype == OT_NULL) {
        if(obj->len()) {
            sq_pushinteger(vm, 0);
        }
		return 1;
    } else if (argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        if ((size_t)index + 1 < obj->len()) {
            sq_pushinteger(vm, index + 1);
            return 1;
        }
    }
    sq_pushnull(vm);
	return 1;
}
//
// IO.Zip entries
//
#define BIND_METHOD_NAME "entries"
SQInteger IOZipentries(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Zip)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->entries())
    // push return value
    BIND_PUSH_INTEGER(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// IO.Zip entry
//
#define BIND_METHOD_NAME "entry"
SQInteger IOZipentry(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Zip)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "path" as string
    BIND_GET_CHECK_STRING(path, 1)
    // return value
    io::Path* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->entry(path))
    // push return value
    pushIOPath(vm, ret);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger IOZipToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(IO.Zip)", 8);
    return 1;
}
#undef BIND_CLASS_NAME


void bindIO(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("IO", 2);

    // create enumeration
    sq_pushstring(vm, "Type", -1);
    sq_newtable(vm);
    sq_pushstring(vm, "Float", -1);
    sq_pushinteger(vm, Type::Float);
    sq_newslot(vm, -3, true);
    sq_pushstring(vm, "Integer", -1);
    sq_pushinteger(vm, Type::Integer);
    sq_newslot(vm, -3, true);
    sq_pushstring(vm, "String", -1);
    sq_pushinteger(vm, Type::String);
    sq_newslot(vm, -3, true);
    sq_newslot(vm, -3, true);
    // create class IO.File
    binding.startClass("File", 4, IOFileObject);
    binding.addMethod("constructor", 11, &IOFileCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<File>); // clone
    binding.addMethod("_get", 4, &IOFileGetters);
    binding.addMethod("_set", 4, &IOFileSetters);
    binding.addMethod("bytes", 5, &IOFilebytes);
    binding.addMethod("exists", 6, &IOFileexists);
    binding.addMethod("folder", 6, &IOFilefolder);
    binding.addMethod("len", 3, &IOFilelen);
    binding.addMethod("readAll", 7, &IOFilereadAll);
    binding.addMethod("reader", 6, &IOFilereader);
    binding.addMethod("remove", 6, &IOFileremove);
    binding.addMethod("text", 4, &IOFiletext);
    binding.addMethod("writeAll", 8, &IOFilewriteAll);
    binding.addMethod("writer", 6, &IOFilewriter);
    binding.addMethod("_tostring", 9, &IOFileToString);
    binding.endClass();

    // create class IO.Folder
    binding.startClass("Folder", 6, IOFolderObject);
    binding.addMethod("constructor", 11, &IOFolderCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Folder>); // clone
    binding.addMethod("_get", 4, &IOFolderGetters);
    binding.addMethod("_set", 4, &IOFolderSetters);
    binding.addMethod("children", 8, &IOFolderchildren);
    binding.addMethod("exists", 6, &IOFolderexists);
    binding.addMethod("folder", 6, &IOFolderfolder);
    binding.addMethod("mkdir", 5, &IOFoldermkdir);
    binding.addMethod("_tostring", 9, &IOFolderToString);
    binding.endClass();

    // create class IO.Json
    binding.startClass("Json", 4, IOJsonObject);
    binding.addMethod("constructor", 11, &IOJsonCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Json>); // clone
    binding.addMethod("_get", 4, &IOJsonGetters);
    binding.addMethod("_set", 4, &IOJsonSetters);
    binding.addMethod("read", 4, &IOJsonread);
    binding.addMethod("write", 5, &IOJsonwrite);
    binding.addMethod("_tostring", 9, &IOJsonToString);
    binding.endClass();

    // create class IO.Xml
    binding.startClass("Xml", 3, IOXmlObject);
    binding.addMethod("constructor", 11, &IOXmlCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Xml>); // clone
    binding.addMethod("_get", 4, &IOXmlGetters);
    binding.addMethod("_set", 4, &IOXmlSetters);
    binding.addMethod("read", 4, &IOXmlread);
    binding.addMethod("_tostring", 9, &IOXmlToString);
    binding.endClass();

    // create class IO.ZipFile
    binding.startClass("ZipFile", 7, IOZipFileObject);
    binding.addMethod("constructor", 11, &IOZipFileCtor); // ctor
    binding.addMethod("_cloned", 7, &IOZipFileNoclone); // unclonable
    binding.addMethod("_get", 4, &IOZipFileGetters);
    binding.addMethod("_set", 4, &IOZipFileSetters);
    binding.addMethod("bytes", 5, &IOZipFilebytes);
    binding.addMethod("exists", 6, &IOZipFileexists);
    binding.addMethod("folder", 6, &IOZipFilefolder);
    binding.addMethod("size", 4, &IOZipFilesize);
    binding.addMethod("text", 4, &IOZipFiletext);
    binding.addMethod("_tostring", 9, &IOZipFileToString);
    binding.endClass();

    // create class IO.ZipFolder
    binding.startClass("ZipFolder", 9, IOZipFolderObject);
    binding.addMethod("constructor", 11, &IOZipFolderCtor); // ctor
    binding.addMethod("_cloned", 7, &IOZipFolderNoclone); // unclonable
    binding.addMethod("_get", 4, &IOZipFolderGetters);
    binding.addMethod("_set", 4, &IOZipFolderSetters);
    binding.addMethod("exists", 6, &IOZipFolderexists);
    binding.addMethod("folder", 6, &IOZipFolderfolder);
    binding.addMethod("_tostring", 9, &IOZipFolderToString);
    binding.endClass();

    // create class IO.Zip
    binding.startClass("Zip", 3, IOZipObject);
    binding.addMethod("constructor", 11, &IOZipCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Zip>); // clone
    binding.addMethod("_get", 4, &IOZipGetters);
    binding.addMethod("_nexti", 6, &IOZipNexti);
    binding.addMethod("entries", 7, &IOZipentries);
    binding.addMethod("entry", 5, &IOZipentry);
    binding.addMethod("_tostring", 9, &IOZipToString);
    binding.endClass();

    binding.endTable(); // end package table
}
}}
