/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "bindmidi.h"
#include "bindinterface.h"
#include "bindutil.h"
#include "bindtime.h"
#include "bindio.h"

#include "abc.h"
#include "beattracker.h"
#include "midibeatclock.h"
#include "midifile.h"
#include "midiport.h"
#include "module_midi.h"
#include "mml.h"
#include "midisequence.h"
#include "tab.h"
#include "midipackage.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

using namespace midi;

namespace binding {

// object references to types in this package
HSQOBJECT MidiAbcObject;
HSQOBJECT MidiBeatTrackerObject;
HSQOBJECT MidiClockGeneratorObject;
HSQOBJECT MidiConnectionObject;
HSQOBJECT MidiConnectorObject;
HSQOBJECT MidiCopyrightObject;
HSQOBJECT MidiCuePointObject;
HSQOBJECT MidiMessageEventObject;
HSQOBJECT MidiFileObject;
HSQOBJECT MidiInputObject;
HSQOBJECT MidiInstrumentObject;
HSQOBJECT MidiLyricObject;
HSQOBJECT MidiMarkerObject;
HSQOBJECT MidiNoteObject;
HSQOBJECT MidiNoteEventObject;
HSQOBJECT MidiNoteOnObject;
HSQOBJECT MidiNoteOffObject;
HSQOBJECT MidiKeySignatureObject;
HSQOBJECT MidiControlObject;
HSQOBJECT MidiMMLObject;
HSQOBJECT MidiPatternObject;
HSQOBJECT MidiSequenceObject;
HSQOBJECT MidiTempoChangeObject;
HSQOBJECT MidiTextObject;
HSQOBJECT MidiTimeSignatureObject;
HSQOBJECT MidiTrackNameObject;
HSQOBJECT MidiOutputObject;
HSQOBJECT MidiPitchBendObject;
HSQOBJECT MidiProgramChangeObject;
HSQOBJECT MidiTabObject;

#define BIND_CLASS_NAME "Midi"
//
// Midi abc
//
#define BIND_METHOD_NAME "abc"
SQInteger Midiabc(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "abc" as string
    BIND_GET_CHECK_STRING(abc, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = Midi::abc(abc))
    // push return value
    sq_pushobject(vm, MidiSequenceObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Sequence>);
    return 1;
}
#undef BIND_METHOD_NAME

#undef BIND_CLASS_NAME
//
// Midi.Abc class
//
#define BIND_CLASS_NAME "Abc"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiAbcCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Abc *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Abc())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Abc>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Abc abc
//
#define BIND_METHOD_NAME "abc"
SQInteger MidiAbcabc(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Abc)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "note" as Midi.Note
    BIND_GET_CHECK_OBJECT(note, 1, Midi, Note, midi::Note)
    // return value
    std::string ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->abc(*note))
    // push return value
    BIND_PUSH_STRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Abc file
//
#define BIND_METHOD_NAME "file"
SQInteger MidiAbcfile(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Abc)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "abc" as string
    BIND_GET_CHECK_STRING(abc, 1)
    // return value
    midi::File* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->file(abc))
    // push return value
    sq_pushobject(vm, MidiFileObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::File>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Abc sequence
//
#define BIND_METHOD_NAME "sequence"
SQInteger MidiAbcsequence(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Abc)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "abc" as string
    BIND_GET_CHECK_STRING(abc, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->sequence(abc))
    // push return value
    sq_pushobject(vm, MidiSequenceObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Sequence>);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MidiAbcToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Abc)", 10);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.BeatTracker class
//
#define BIND_CLASS_NAME "BeatTracker"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiBeatTrackerCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BeatTracker *obj;
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "connection" as Midi.Input
    BIND_GET_CHECK_OBJECT(connection, 1, Midi, Input, midi::MidiInputPort)
    // get parameter 2 "bpm" as float
    BIND_GET_CHECK_FLOAT(bpm, 2)
    // call the implementation
    BIND_WRAP_TRY(obj = BeatTrackerFactory::instance().newBeatTracker(*connection, bpm))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MidiBeatTrackerNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.BeatTracker");
}
//
// Midi.BeatTracker start
//
#define BIND_METHOD_NAME "start"
SQInteger MidiBeatTrackerstart(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BeatTracker)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->start())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.BeatTracker stop
//
#define BIND_METHOD_NAME "stop"
SQInteger MidiBeatTrackerstop(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(BeatTracker)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->stop())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger MidiBeatTrackerToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.BeatTracker)", 18);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.ClockGenerator class
//
#define BIND_CLASS_NAME "ClockGenerator"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiClockGeneratorCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    ClockGenerator *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = midi::ClockGeneratorFactory::instance().newMidiClockGenerator())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MidiClockGeneratorNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.ClockGenerator");
}
//
// Midi.ClockGenerator property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiClockGeneratorGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::ClockGenerator)
    // dispatch appropriate getter
    if(!strcmp(key, "midiout")) {
        HSQOBJECT ret = obj->midiout();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.ClockGenerator property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiClockGeneratorSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::ClockGenerator)
    // dispatch appropriate setter
    if(!strcmp(key, "midiout")) {
        return sq_throwerror(vm, "property \"midiout\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiClockGeneratorToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.ClockGenerator)", 21);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Connection class
//
#define BIND_CLASS_NAME "Connection"
SQInteger MidiConnectionCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Midi.Connection");
}

SQInteger MidiConnectionNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.Connection");
}
//
// Midi.Connection onControl
//
#define BIND_METHOD_NAME "onControl"
SQInteger MidiConnectiononControl(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onControl(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connection onNoteOff
//
#define BIND_METHOD_NAME "onNoteOff"
SQInteger MidiConnectiononNoteOff(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onNoteOff(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connection onNoteOn
//
#define BIND_METHOD_NAME "onNoteOn"
SQInteger MidiConnectiononNoteOn(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onNoteOn(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connection onProgramChange
//
#define BIND_METHOD_NAME "onProgramChange"
SQInteger MidiConnectiononProgramChange(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connection)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onProgramChange(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger MidiConnectionToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Connection)", 17);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Connector class
//
#define BIND_CLASS_NAME "Connector"
SQInteger MidiConnectorCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Midi.Connector");
}

SQInteger MidiConnectorNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.Connector");
}
//
// Midi.Connector clear
//
#define BIND_METHOD_NAME "clear"
SQInteger MidiConnectorclear(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj->clear())
    } else {
        // get parameter 1 "clock" as Time.Clock
        BIND_GET_CHECK_INTERFACE(clock, 1, Time, Clock, time::Clock)
        // call the implementation
        BIND_WRAP_TRY(obj->clear(*clock))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Connector schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger MidiConnectorschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Connector)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &MidiNoteObject) {
        // get parameter 1 "buffer" as Midi.Note
        BIND_GET_OBJECT(buffer, 1, Midi, Note, midi::Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*buffer, measure, *clock))
        }
    }
    else if(arg1type == &MidiPatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi::Pattern)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*pattern, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*pattern, measure, *clock))
        }
    }
    else if(midi::Message *message = getMidiMessage(arg1type, &arg1)) {
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*message, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*message, measure, *clock))
        }
    }
    else if(arg1type == &MidiSequenceObject) {
        // get parameter 1 "message" as Midi.Sequence
        BIND_GET_OBJECT(message, 1, Midi, Sequence, midi::Sequence)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*message, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*message, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger MidiConnectorToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Connector)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Copyright class
//
#define BIND_CLASS_NAME "Copyright"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiCopyrightCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Copyright *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Copyright(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Copyright>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Copyright property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiCopyrightGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Copyright)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Copyright property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiCopyrightSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Copyright)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiCopyrightToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Copyright)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.CuePoint class
//
#define BIND_CLASS_NAME "CuePoint"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiCuePointCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    CuePoint *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new CuePoint(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<CuePoint>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.CuePoint property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiCuePointGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::CuePoint)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.CuePoint property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiCuePointSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::CuePoint)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiCuePointToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.CuePoint)", 15);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.MessageEvent class
//
#define BIND_CLASS_NAME "MessageEvent"
SQInteger MidiMessageEventCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Midi.MessageEvent");
}

SQInteger MidiMessageEventNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.MessageEvent");
}
//
// Midi.MessageEvent property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiMessageEventGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MessageEvent)
    // dispatch appropriate getter
    if(!strcmp(key, "message")) {
        midi::Message* ret = obj->message();
        pushMidiMessage(vm, ret);
        return 1;
    }
    if(!strcmp(key, "measure")) {
        SQFloat ret = obj->measure();
        BIND_PUSH_FLOAT(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.MessageEvent property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiMessageEventSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MessageEvent)
    // dispatch appropriate setter
    if(!strcmp(key, "message")) {
        return sq_throwerror(vm, "property \"message\" is read-only");
    }
    if(!strcmp(key, "measure")) {
        return sq_throwerror(vm, "property \"measure\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiMessageEventToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.MessageEvent)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.File class
//
#define BIND_CLASS_NAME "File"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiFileCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    File *obj;
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj = new File())
    } else {
        HSQOBJECT arg1;
        sq_getstackobj(vm, 2, &arg1);
        if(arg1._type == OT_STRING) {
            BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
            // get parameter 1 "path" as string
            BIND_GET_STRING(path, 1)
            // call the implementation
            BIND_WRAP_TRY(obj = new File(path))
        }
        else if(arg1._type == OT_INSTANCE) {
        SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
        if(arg1type == &IOFileObject) {
            // get parameter 1 "file" as IO.File
            BIND_GET_OBJECT(file, 1, IO, File, io::File)
            // call the implementation
            BIND_WRAP_TRY(obj = new File(*file))
        }
        else {
            return sq_throwerror(vm, "argument 1 is not of type {IO.File, string}");
        }
        }
        else {
            return sq_throwerror(vm, "argument 1 is not of type {IO.File, string}");
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<File>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.File property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiFileGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::File)
    // dispatch appropriate getter
    if(!strcmp(key, "track")) {
        HSQOBJECT ret = obj->track();
        sq_pushobject(vm, ret);
        return 1;
    }
    if(!strcmp(key, "ppq")) {
        SQInteger ret = obj->ppq();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "type")) {
        SQInteger ret = obj->type();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.File property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiFileSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::File)
    // dispatch appropriate setter
    if(!strcmp(key, "track")) {
        // get parameter 2 "track" as array
        BIND_GET_CHECK_ARRAY(track, 2)
        BIND_WRAP_TRY(obj->track(track))
        return 0;
    }
    if(!strcmp(key, "ppq")) {
        // get parameter 2 "ppq" as integer
        BIND_GET_CHECK_INTEGER(ppq, 2)
        BIND_WRAP_TRY(obj->ppq(ppq))
        return 0;
    }
    if(!strcmp(key, "type")) {
        // get parameter 2 "type" as integer
        BIND_GET_CHECK_INTEGER(type, 2)
        BIND_WRAP_TRY(obj->type(type))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.File save
//
#define BIND_METHOD_NAME "save"
SQInteger MidiFilesave(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_STRING) {
        // get parameter 1 "path" as string
        BIND_GET_STRING(path, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->save(path))
    }
    else if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &IOFileObject) {
        // get parameter 1 "file" as IO.File
        BIND_GET_OBJECT(file, 1, IO, File, io::File)
        // call the implementation
        BIND_WRAP_TRY(obj->save(*file))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {IO.File, string}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {IO.File, string}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.File tracks
//
#define BIND_METHOD_NAME "tracks"
SQInteger MidiFiletracks(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(File)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->tracks())
    // push return value
    BIND_PUSH_INTEGER(ret);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MidiFileToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.File)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Input class
//
#define BIND_CLASS_NAME "Input"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiInputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MidiInputPort *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = MidiInputPortFactory::instance().newMidiInputPort(name))
    } else {
        HSQOBJECT arg2;
        sq_getstackobj(vm, 3, &arg2);
        if(arg2._type == OT_STRING) {
            // get parameter 2 "connection" as string
            BIND_GET_STRING(connection, 2)
            // call the implementation
            BIND_WRAP_TRY(obj = MidiInputPortFactory::instance().newMidiInputPort(name, connection))
        }
        else if(arg2._type == OT_INSTANCE) {
        SQUserPointer arg2type = bip_getinstancetypetag(&arg2);
        if(arg2type == &regexObject) {
            BIND_NARGS_CHECK_MIN(2) // check min parameter count
            // get parameter 2 "connection" as regex
            BIND_GET_OBJECT(connection, 2, , regex, bipscript::Regex)
            // call the implementation
            BIND_WRAP_TRY(obj = MidiInputPortFactory::instance().newMidiInputPort(name, *connection))
        }
        else {
            return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
        }
        }
        else {
            return sq_throwerror(vm, "argument 2 is not of type {string, regex}");
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MidiInputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.Input");
}
//
// Midi.Input property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiInputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MidiInputPort)
    // dispatch appropriate getter
    if(!strcmp(key, "midiout")) {
        HSQOBJECT ret = obj->midiout();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Input property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiInputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MidiInputPort)
    // dispatch appropriate setter
    if(!strcmp(key, "midiout")) {
        return sq_throwerror(vm, "property \"midiout\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Input onControl
//
#define BIND_METHOD_NAME "onControl"
SQInteger MidiInputonControl(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onControl(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input onNoteOff
//
#define BIND_METHOD_NAME "onNoteOff"
SQInteger MidiInputonNoteOff(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onNoteOff(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input onNoteOn
//
#define BIND_METHOD_NAME "onNoteOn"
SQInteger MidiInputonNoteOn(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onNoteOn(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Input onProgramChange
//
#define BIND_METHOD_NAME "onProgramChange"
SQInteger MidiInputonProgramChange(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiInputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onProgramChange(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger MidiInputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Input)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Instrument class
//
#define BIND_CLASS_NAME "Instrument"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiInstrumentCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Instrument *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Instrument(name))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Instrument>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Instrument property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiInstrumentGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Instrument)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Instrument property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiInstrumentSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Instrument)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiInstrumentToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Instrument)", 17);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Lyric class
//
#define BIND_CLASS_NAME "Lyric"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiLyricCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Lyric *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Lyric(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Lyric>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Lyric property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiLyricGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Lyric)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Lyric property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiLyricSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Lyric)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiLyricToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Lyric)", 12);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Marker class
//
#define BIND_CLASS_NAME "Marker"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiMarkerCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Marker *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Marker(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Marker>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Marker property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiMarkerGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Marker)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Marker property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiMarkerSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Marker)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiMarkerToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Marker)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Note class
//
#define BIND_CLASS_NAME "Note"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiNoteCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Note *obj;
    BIND_NARGS_CHECK_MIN(3) // check min parameter count
    BIND_NARGS_CHECK_MAX(4) // check max parameter count
    // get parameter 1 "pitch" as integer
    BIND_GET_CHECK_INTEGER(pitch, 1)
    // get parameter 2 "velocity" as integer
    BIND_GET_CHECK_INTEGER(velocity, 2)
    // get parameter 3 "duration" as float
    BIND_GET_CHECK_FLOAT(duration, 3)
    if(numargs == 4) {
        // call the implementation
        BIND_WRAP_TRY(obj = new Note(pitch, velocity, duration))
    } else {
        // get parameter 4 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 4)
        // call the implementation
        BIND_WRAP_TRY(obj = new Note(pitch, velocity, duration, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Note>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Note property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiNoteGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Note)
    // dispatch appropriate getter
    if(!strcmp(key, "pitch")) {
        SQInteger ret = obj->pitch();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "velocity")) {
        SQInteger ret = obj->velocity();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "duration")) {
        SQFloat ret = obj->duration();
        BIND_PUSH_FLOAT(ret);
        return 1;
    }
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Note property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiNoteSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Note)
    // dispatch appropriate setter
    if(!strcmp(key, "pitch")) {
        return sq_throwerror(vm, "property \"pitch\" is read-only");
    }
    if(!strcmp(key, "velocity")) {
        return sq_throwerror(vm, "property \"velocity\" is read-only");
    }
    if(!strcmp(key, "duration")) {
        return sq_throwerror(vm, "property \"duration\" is read-only");
    }
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiNoteToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Note)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.NoteEvent class
//
#define BIND_CLASS_NAME "NoteEvent"
SQInteger MidiNoteEventCtor(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot directly instantiate Midi.NoteEvent");
}

SQInteger MidiNoteEventNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.NoteEvent");
}
//
// Midi.NoteEvent property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiNoteEventGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteEvent)
    // dispatch appropriate getter
    if(!strcmp(key, "note")) {
        midi::Note* ret = obj->note();
        sq_pushobject(vm, MidiNoteObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::Note>);
        return 1;
    }
    if(!strcmp(key, "measure")) {
        SQFloat ret = obj->measure();
        BIND_PUSH_FLOAT(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.NoteEvent property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiNoteEventSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteEvent)
    // dispatch appropriate setter
    if(!strcmp(key, "note")) {
        return sq_throwerror(vm, "property \"note\" is read-only");
    }
    if(!strcmp(key, "measure")) {
        return sq_throwerror(vm, "property \"measure\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiNoteEventToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.NoteEvent)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.NoteOn class
//
#define BIND_CLASS_NAME "NoteOn"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiNoteOnCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    NoteOn *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "pitch" as integer
    BIND_GET_CHECK_INTEGER(pitch, 1)
    // get parameter 2 "velocity" as integer
    BIND_GET_CHECK_INTEGER(velocity, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = new NoteOn(pitch, velocity))
    } else {
        // get parameter 3 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = new NoteOn(pitch, velocity, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<NoteOn>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.NoteOn property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiNoteOnGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteOn)
    // dispatch appropriate getter
    if(!strcmp(key, "pitch")) {
        SQInteger ret = obj->pitch();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "velocity")) {
        SQInteger ret = obj->velocity();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.NoteOn property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiNoteOnSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteOn)
    // dispatch appropriate setter
    if(!strcmp(key, "pitch")) {
        return sq_throwerror(vm, "property \"pitch\" is read-only");
    }
    if(!strcmp(key, "velocity")) {
        return sq_throwerror(vm, "property \"velocity\" is read-only");
    }
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiNoteOnToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.NoteOn)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.NoteOff class
//
#define BIND_CLASS_NAME "NoteOff"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiNoteOffCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    NoteOff *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "pitch" as integer
    BIND_GET_CHECK_INTEGER(pitch, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = new NoteOff(pitch))
    } else {
        // get parameter 2 "velocity" as integer
        BIND_GET_CHECK_INTEGER(velocity, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj = new NoteOff(pitch, velocity))
        } else {
            // get parameter 3 "channel" as integer
            BIND_GET_CHECK_INTEGER(channel, 3)
            // call the implementation
            BIND_WRAP_TRY(obj = new NoteOff(pitch, velocity, channel))
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<NoteOff>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.NoteOff property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiNoteOffGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteOff)
    // dispatch appropriate getter
    if(!strcmp(key, "pitch")) {
        SQInteger ret = obj->pitch();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "velocity")) {
        SQInteger ret = obj->velocity();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.NoteOff property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiNoteOffSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::NoteOff)
    // dispatch appropriate setter
    if(!strcmp(key, "pitch")) {
        return sq_throwerror(vm, "property \"pitch\" is read-only");
    }
    if(!strcmp(key, "velocity")) {
        return sq_throwerror(vm, "property \"velocity\" is read-only");
    }
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiNoteOffToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.NoteOff)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.KeySignature class
//
#define BIND_CLASS_NAME "KeySignature"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiKeySignatureCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    KeySignature *obj;
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "count" as integer
    BIND_GET_CHECK_INTEGER(count, 1)
    // get parameter 2 "minor" as bool
    BIND_GET_CHECK_BOOL(minor, 2)
    // call the implementation
    BIND_WRAP_TRY(obj = new KeySignature(count, minor))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<KeySignature>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.KeySignature property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiKeySignatureGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::KeySignature)
    // dispatch appropriate getter
    if(!strcmp(key, "count")) {
        SQInteger ret = obj->count();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "minor")) {
        SQBool ret = obj->minor();
        BIND_PUSH_BOOL(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.KeySignature property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiKeySignatureSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::KeySignature)
    // dispatch appropriate setter
    if(!strcmp(key, "count")) {
        return sq_throwerror(vm, "property \"count\" is read-only");
    }
    if(!strcmp(key, "minor")) {
        return sq_throwerror(vm, "property \"minor\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiKeySignatureToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.KeySignature)", 19);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Control class
//
#define BIND_CLASS_NAME "Control"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiControlCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Control *obj;
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "control" as integer
    BIND_GET_CHECK_INTEGER(control, 1)
    // get parameter 2 "value" as integer
    BIND_GET_CHECK_INTEGER(value, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj = new Control(control, value))
    } else {
        // get parameter 3 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 3)
        // call the implementation
        BIND_WRAP_TRY(obj = new Control(control, value, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Control>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Control property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiControlGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Control)
    // dispatch appropriate getter
    if(!strcmp(key, "controller")) {
        SQInteger ret = obj->controller();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "value")) {
        SQInteger ret = obj->value();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Control property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiControlSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Control)
    // dispatch appropriate setter
    if(!strcmp(key, "controller")) {
        return sq_throwerror(vm, "property \"controller\" is read-only");
    }
    if(!strcmp(key, "value")) {
        return sq_throwerror(vm, "property \"value\" is read-only");
    }
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiControlToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Control)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.MML class
//
#define BIND_CLASS_NAME "MML"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiMMLCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MML *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new MML())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<MML>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.MML sequence
//
#define BIND_METHOD_NAME "sequence"
SQInteger MidiMMLsequence(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MML)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "mml" as string
    BIND_GET_CHECK_STRING(mml, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->sequence(mml))
    // push return value
    sq_pushobject(vm, MidiSequenceObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Sequence>);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MidiMMLToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.MML)", 10);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Pattern class
//
#define BIND_CLASS_NAME "Pattern"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiPatternCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Pattern *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Pattern())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Pattern>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiPatternGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        BIND_GET_THIS_PROPERTY(midi::Pattern)
        midi::NoteEvent* ret = (*obj)[index];
        if(ret) {
            sq_pushobject(vm, MidiNoteEventObject);
            sq_createinstance(vm, -1);
            sq_remove(vm, -2);
            sq_setinstanceup(vm, -1, ret->addref());
            sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::NoteEvent>);
            return 1;
        }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME
//
// Midi.Pattern _nexti
//
SQInteger MidiPatternNexti(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
	BIND_GET_THIS_PROPERTY(Pattern)
    if (argtype == OT_NULL) {
        if(obj->len()) {
            sq_pushinteger(vm, 0);
        }
		return 1;
    } else if (argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        if ((size_t)index + 1 < obj->len()) {
            sq_pushinteger(vm, index + 1);
            return 1;
        }
    }
    sq_pushnull(vm);
	return 1;
}
//
// Midi.Pattern add
//
#define BIND_METHOD_NAME "add"
SQInteger MidiPatternadd(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &MidiNoteObject) {
        BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
        // get parameter 1 "note" as Midi.Note
        BIND_GET_OBJECT(note, 1, Midi, Note, midi::Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->add(*note, measure))
    }
    else if(arg1type == &MidiPatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi::Pattern)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern))
        } else {
            // get parameter 2 "measure" as float
            BIND_GET_CHECK_FLOAT(measure, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern, measure))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern filter
//
#define BIND_METHOD_NAME "filter"
SQInteger MidiPatternfilter(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "filter" as function
    BIND_GET_CHECK_FUNCTION(filter, 1)
    // return value
    midi::Pattern* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->filter(filter))
    // push return value
    sq_pushobject(vm, MidiPatternObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Pattern>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern len
//
#define BIND_METHOD_NAME "len"
SQInteger MidiPatternlen(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->len())
    // push return value
    BIND_PUSH_INTEGER(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern print
//
#define BIND_METHOD_NAME "print"
SQInteger MidiPatternprint(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj->print())
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern sequence
//
#define BIND_METHOD_NAME "sequence"
SQInteger MidiPatternsequence(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->sequence())
    // push return value
    sq_pushobject(vm, MidiSequenceObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Sequence>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern slice
//
#define BIND_METHOD_NAME "slice"
SQInteger MidiPatternslice(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "start" as float
    BIND_GET_CHECK_FLOAT(start, 1)
    // get parameter 2 "end" as float
    BIND_GET_CHECK_FLOAT(end, 2)
    // return value
    midi::Pattern* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->slice(start, end))
    // push return value
    sq_pushobject(vm, MidiPatternObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Pattern>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Pattern transpose
//
#define BIND_METHOD_NAME "transpose"
SQInteger MidiPatterntranspose(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Pattern)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "amount" as integer
    BIND_GET_CHECK_INTEGER(amount, 1)
    // return value
    midi::Pattern* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->transpose(amount))
    // push return value
    sq_pushobject(vm, MidiPatternObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Pattern>);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MidiPatternToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Pattern)", 14);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Sequence class
//
#define BIND_CLASS_NAME "Sequence"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiSequenceCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Sequence *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Sequence())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Sequence>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Sequence property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiSequenceGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        BIND_GET_THIS_PROPERTY(midi::Sequence)
        midi::MessageEvent* ret = (*obj)[index];
        if(ret) {
            sq_pushobject(vm, MidiMessageEventObject);
            sq_createinstance(vm, -1);
            sq_remove(vm, -2);
            sq_setinstanceup(vm, -1, ret->addref());
            sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::MessageEvent>);
            return 1;
        }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME
//
// Midi.Sequence _nexti
//
SQInteger MidiSequenceNexti(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
	BIND_GET_THIS_PROPERTY(Sequence)
    if (argtype == OT_NULL) {
        if(obj->len()) {
            sq_pushinteger(vm, 0);
        }
		return 1;
    } else if (argtype == OT_INTEGER) {
        SQInteger index;
        sq_getinteger(vm, 2, &index);
        if ((size_t)index + 1 < obj->len()) {
            sq_pushinteger(vm, index + 1);
            return 1;
        }
    }
    sq_pushnull(vm);
	return 1;
}
//
// Midi.Sequence add
//
#define BIND_METHOD_NAME "add"
SQInteger MidiSequenceadd(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Sequence)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(midi::Message *message = getMidiMessage(arg1type, &arg1)) {
        BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->add(*message, measure))
    }
    else if(arg1type == &MidiNoteObject) {
        BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
        // get parameter 1 "message" as Midi.Note
        BIND_GET_OBJECT(message, 1, Midi, Note, midi::Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // call the implementation
        BIND_WRAP_TRY(obj->add(*message, measure))
    }
    else if(arg1type == &MidiPatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi::Pattern)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern))
        } else {
            // get parameter 2 "measure" as float
            BIND_GET_CHECK_FLOAT(measure, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern, measure))
        }
    }
    else if(arg1type == &MidiSequenceObject) {
        // get parameter 1 "pattern" as Midi.Sequence
        BIND_GET_OBJECT(pattern, 1, Midi, Sequence, midi::Sequence)
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern))
        } else {
            // get parameter 2 "measure" as float
            BIND_GET_CHECK_FLOAT(measure, 2)
            // call the implementation
            BIND_WRAP_TRY(obj->add(*pattern, measure))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Sequence filter
//
#define BIND_METHOD_NAME "filter"
SQInteger MidiSequencefilter(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Sequence)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "filter" as function
    BIND_GET_CHECK_FUNCTION(filter, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->filter(filter))
    // push return value
    sq_pushobject(vm, MidiSequenceObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Sequence>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Sequence len
//
#define BIND_METHOD_NAME "len"
SQInteger MidiSequencelen(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Sequence)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->len())
    // push return value
    BIND_PUSH_INTEGER(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Sequence pattern
//
#define BIND_METHOD_NAME "pattern"
SQInteger MidiSequencepattern(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Sequence)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    midi::Pattern* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->pattern())
    // push return value
    sq_pushobject(vm, MidiPatternObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Pattern>);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MidiSequenceToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Sequence)", 15);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.TempoChange class
//
#define BIND_CLASS_NAME "TempoChange"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiTempoChangeCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    TempoChange *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "tempo" as float
    BIND_GET_CHECK_FLOAT(tempo, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new TempoChange(tempo))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<TempoChange>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.TempoChange property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiTempoChangeGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TempoChange)
    // dispatch appropriate getter
    if(!strcmp(key, "tempo")) {
        SQFloat ret = obj->tempo();
        BIND_PUSH_FLOAT(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.TempoChange property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiTempoChangeSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TempoChange)
    // dispatch appropriate setter
    if(!strcmp(key, "tempo")) {
        return sq_throwerror(vm, "property \"tempo\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiTempoChangeToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.TempoChange)", 18);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Text class
//
#define BIND_CLASS_NAME "Text"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiTextCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Text *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "text" as string
    BIND_GET_CHECK_STRING(text, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Text(text))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<Text>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Text property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiTextGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Text)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Text property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiTextSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Text)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiTextToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Text)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.TimeSignature class
//
#define BIND_CLASS_NAME "TimeSignature"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiTimeSignatureCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    TimeSignature *obj;
    BIND_NARGS_CHECK_EXACT(4) // check exact parameter count
    // get parameter 1 "numerator" as integer
    BIND_GET_CHECK_INTEGER(numerator, 1)
    // get parameter 2 "denominator" as integer
    BIND_GET_CHECK_INTEGER(denominator, 2)
    // get parameter 3 "ppq" as integer
    BIND_GET_CHECK_INTEGER(ppq, 3)
    // get parameter 4 "dspq" as integer
    BIND_GET_CHECK_INTEGER(dspq, 4)
    // call the implementation
    BIND_WRAP_TRY(obj = new TimeSignature(numerator, denominator, ppq, dspq))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<TimeSignature>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.TimeSignature property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiTimeSignatureGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TimeSignature)
    // dispatch appropriate getter
    if(!strcmp(key, "numerator")) {
        SQInteger ret = obj->numerator();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "denominator")) {
        SQInteger ret = obj->denominator();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.TimeSignature property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiTimeSignatureSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TimeSignature)
    // dispatch appropriate setter
    if(!strcmp(key, "numerator")) {
        return sq_throwerror(vm, "property \"numerator\" is read-only");
    }
    if(!strcmp(key, "denominator")) {
        return sq_throwerror(vm, "property \"denominator\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiTimeSignatureToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.TimeSignature)", 20);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.TrackName class
//
#define BIND_CLASS_NAME "TrackName"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiTrackNameCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    TrackName *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new TrackName(name))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<TrackName>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.TrackName property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiTrackNameGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TrackName)
    // dispatch appropriate getter
    if(!strcmp(key, "text")) {
        std::string ret = obj->text();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.TrackName property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiTrackNameSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::TrackName)
    // dispatch appropriate setter
    if(!strcmp(key, "text")) {
        return sq_throwerror(vm, "property \"text\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiTrackNameToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.TrackName)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Output class
//
#define BIND_CLASS_NAME "Output"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiOutputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    MidiOutputPort *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "name" as string
    BIND_GET_CHECK_STRING(name, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = MidiOutputPortFactory::instance().newMidiOutputPort(name))
    } else {
        // get parameter 2 "connection" as string
        BIND_GET_CHECK_STRING(connection, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = MidiOutputPortFactory::instance().newMidiOutputPort(name, connection))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger MidiOutputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Midi.Output");
}
//
// Midi.Output property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiOutputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MidiOutputPort)
    // dispatch appropriate getter
    if(!strcmp(key, "midiin")) {
        HSQOBJECT ret = obj->midiin();
        sq_pushobject(vm, ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Output property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiOutputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::MidiOutputPort)
    // dispatch appropriate setter
    if(!strcmp(key, "midiin")) {
        return sq_throwerror(vm, "property \"midiin\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Output clear
//
#define BIND_METHOD_NAME "clear"
SQInteger MidiOutputclear(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_MAX(1) // check max parameter count
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj->clear())
    } else {
        // get parameter 1 "clock" as Time.Clock
        BIND_GET_CHECK_INTERFACE(clock, 1, Time, Clock, time::Clock)
        // call the implementation
        BIND_WRAP_TRY(obj->clear(*clock))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Output remove
//
#define BIND_METHOD_NAME "remove"
SQInteger MidiOutputremove(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &MidiMessageEventObject) {
        // get parameter 1 "event" as Midi.MessageEvent
        BIND_GET_OBJECT(event, 1, Midi, MessageEvent, midi::MessageEvent)
        // call the implementation
        BIND_WRAP_TRY(obj->remove(*event))
    }
    else if(arg1type == &MidiNoteEventObject) {
        // get parameter 1 "event" as Midi.NoteEvent
        BIND_GET_OBJECT(event, 1, Midi, NoteEvent, midi::NoteEvent)
        // call the implementation
        BIND_WRAP_TRY(obj->remove(*event))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.MessageEvent, Midi.NoteEvent}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.MessageEvent, Midi.NoteEvent}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Output schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger MidiOutputschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &MidiNoteObject) {
        // get parameter 1 "note" as Midi.Note
        BIND_GET_OBJECT(note, 1, Midi, Note, midi::Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // return value
        midi::NoteEvent* ret;
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*note, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*note, measure, *clock))
        }
        // push return value
        sq_pushobject(vm, MidiNoteEventObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::NoteEvent>);
        return 1;

    }
    else if(arg1type == &MidiPatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi::Pattern)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*pattern, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*pattern, measure, *clock))
        }
        // void method, returns no value
        return 0;

    }
    else if(midi::Message *message = getMidiMessage(arg1type, &arg1)) {
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        // return value
        midi::MessageEvent* ret;
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*message, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(ret = obj->schedule(*message, measure, *clock))
        }
        // push return value
        sq_pushobject(vm, MidiMessageEventObject);
        sq_createinstance(vm, -1);
        sq_remove(vm, -2);
        sq_setinstanceup(vm, -1, ret->addref());
        sq_setreleasehook(vm, -1, BindUtil::Decrement<midi::MessageEvent>);
        return 1;

    }
    else if(arg1type == &MidiSequenceObject) {
        // get parameter 1 "sequence" as Midi.Sequence
        BIND_GET_OBJECT(sequence, 1, Midi, Sequence, midi::Sequence)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*sequence, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->schedule(*sequence, measure, *clock))
        }
        // void method, returns no value
        return 0;

    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Note, Midi.Pattern, Midi.Message, Midi.Sequence}");
    }}
#undef BIND_METHOD_NAME

//
// Midi.Output send
//
#define BIND_METHOD_NAME "send"
SQInteger MidiOutputsend(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(MidiOutputPort)
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(midi::Message *message = getMidiMessage(arg1type, &arg1)) {
        if(numargs == 2) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*message))
        } else {
            // get parameter 2 "measure" as float
            BIND_GET_CHECK_FLOAT(measure, 2)
            if(numargs == 3) {
                // call the implementation
                BIND_WRAP_TRY(obj->send(*message, measure))
            } else {
                // get parameter 3 "clock" as Time.Clock
                BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
                // call the implementation
                BIND_WRAP_TRY(obj->send(*message, measure, *clock))
            }
        }
    }
    else if(arg1type == &MidiNoteObject) {
        BIND_NARGS_CHECK_MIN(2) // check min parameter count
        // get parameter 1 "note" as Midi.Note
        BIND_GET_OBJECT(note, 1, Midi, Note, midi::Note)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*note, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->send(*note, measure, *clock))
        }
    }
    else if(arg1type == &MidiPatternObject) {
        BIND_NARGS_CHECK_MIN(2) // check min parameter count
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi::Pattern)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*pattern, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->send(*pattern, measure, *clock))
        }
    }
    else if(arg1type == &MidiSequenceObject) {
        BIND_NARGS_CHECK_MIN(2) // check min parameter count
        // get parameter 1 "sequence" as Midi.Sequence
        BIND_GET_OBJECT(sequence, 1, Midi, Sequence, midi::Sequence)
        // get parameter 2 "measure" as float
        BIND_GET_CHECK_FLOAT(measure, 2)
        if(numargs == 3) {
            // call the implementation
            BIND_WRAP_TRY(obj->send(*sequence, measure))
        } else {
            // get parameter 3 "clock" as Time.Clock
            BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
            // call the implementation
            BIND_WRAP_TRY(obj->send(*sequence, measure, *clock))
        }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Message, Midi.Note, Midi.Pattern, Midi.Sequence}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger MidiOutputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Output)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.PitchBend class
//
#define BIND_CLASS_NAME "PitchBend"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiPitchBendCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    PitchBend *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "amount" as integer
    BIND_GET_CHECK_INTEGER(amount, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = new PitchBend(amount))
    } else {
        // get parameter 2 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = new PitchBend(amount, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<PitchBend>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.PitchBend property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiPitchBendGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::PitchBend)
    // dispatch appropriate getter
    if(!strcmp(key, "value")) {
        SQInteger ret = obj->value();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.PitchBend property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiPitchBendSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::PitchBend)
    // dispatch appropriate setter
    if(!strcmp(key, "value")) {
        return sq_throwerror(vm, "property \"value\" is read-only");
    }
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiPitchBendToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.PitchBend)", 16);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.ProgramChange class
//
#define BIND_CLASS_NAME "ProgramChange"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiProgramChangeCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    ProgramChange *obj;
    BIND_NARGS_CHECK_MIN(1) // check min parameter count
    BIND_NARGS_CHECK_MAX(2) // check max parameter count
    // get parameter 1 "program" as integer
    BIND_GET_CHECK_INTEGER(program, 1)
    if(numargs == 2) {
        // call the implementation
        BIND_WRAP_TRY(obj = new ProgramChange(program))
    } else {
        // get parameter 2 "channel" as integer
        BIND_GET_CHECK_INTEGER(channel, 2)
        // call the implementation
        BIND_WRAP_TRY(obj = new ProgramChange(program, channel))
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj->addref());
    sq_setreleasehook(vm, 1, BindUtil::Decrement<ProgramChange>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.ProgramChange property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiProgramChangeGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::ProgramChange)
    // dispatch appropriate getter
    if(!strcmp(key, "program")) {
        SQInteger ret = obj->program();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "channel")) {
        SQInteger ret = obj->channel();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.ProgramChange property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiProgramChangeSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::ProgramChange)
    // dispatch appropriate setter
    if(!strcmp(key, "program")) {
        return sq_throwerror(vm, "property \"program\" is read-only");
    }
    if(!strcmp(key, "channel")) {
        return sq_throwerror(vm, "property \"channel\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger MidiProgramChangeToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.ProgramChange)", 20);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Midi.Tab class
//
#define BIND_CLASS_NAME "Tab"
#define BIND_METHOD_NAME "constructor"
SQInteger MidiTabCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Tab *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Tab())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Tab>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Tab property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger MidiTabGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Tab)
    // dispatch appropriate getter
    if(!strcmp(key, "division")) {
        SQInteger ret = obj->division();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "linewrap")) {
        SQInteger ret = obj->linewrap();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Tab property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger MidiTabSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(midi::Tab)
    // dispatch appropriate setter
    if(!strcmp(key, "division")) {
        // get parameter 2 "division" as integer
        BIND_GET_CHECK_INTEGER(division, 2)
        BIND_WRAP_TRY(obj->division(division))
        return 0;
    }
    if(!strcmp(key, "linewrap")) {
        // get parameter 2 "linewrap" as integer
        BIND_GET_CHECK_INTEGER(linewrap, 2)
        BIND_WRAP_TRY(obj->linewrap(linewrap))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Midi.Tab define
//
#define BIND_METHOD_NAME "define"
SQInteger MidiTabdefine(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Tab)
    BIND_NARGS_CHECK_EXACT(4) // check exact parameter count
    // get parameter 1 "prefix" as string
    BIND_GET_CHECK_STRING(prefix, 1)
    // get parameter 2 "strike" as string
    BIND_GET_CHECK_STRING(strike, 2)
    // get parameter 3 "pitch" as integer
    BIND_GET_CHECK_INTEGER(pitch, 3)
    // get parameter 4 "velocity" as integer
    BIND_GET_CHECK_INTEGER(velocity, 4)
    // call the implementation
    BIND_WRAP_TRY(obj->define(prefix, strike, pitch, velocity))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Midi.Tab sequence
//
#define BIND_METHOD_NAME "sequence"
SQInteger MidiTabsequence(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Tab)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "tab" as string
    BIND_GET_CHECK_STRING(tab, 1)
    // return value
    midi::Sequence* ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->sequence(tab))
    // push return value
    sq_pushobject(vm, MidiSequenceObject);
    sq_createinstance(vm, -1);
    sq_remove(vm, -2);
    sq_setinstanceup(vm, -1, ret);
    sq_setreleasehook(vm, -1, BindUtil::Delete<midi::Sequence>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Tab string
//
#define BIND_METHOD_NAME "string"
SQInteger MidiTabstring(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Tab)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // return value
    std::string ret;
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INSTANCE) {
    SQUserPointer arg1type = bip_getinstancetypetag(&arg1);
    if(arg1type == &MidiPatternObject) {
        // get parameter 1 "pattern" as Midi.Pattern
        BIND_GET_OBJECT(pattern, 1, Midi, Pattern, midi::Pattern)
        // call the implementation
        BIND_WRAP_TRY(ret = obj->string(*pattern))
    }
    else if(arg1type == &MidiSequenceObject) {
        // get parameter 1 "sequence" as Midi.Sequence
        BIND_GET_OBJECT(sequence, 1, Midi, Sequence, midi::Sequence)
        // call the implementation
        BIND_WRAP_TRY(ret = obj->string(*sequence))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Pattern, Midi.Sequence}");
    }
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {Midi.Pattern, Midi.Sequence}");
    }
    // push return value
    BIND_PUSH_STRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Midi.Tab velocity
//
#define BIND_METHOD_NAME "velocity"
SQInteger MidiTabvelocity(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Tab)
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "strike" as string
    BIND_GET_CHECK_STRING(strike, 1)
    // get parameter 2 "velocity" as integer
    BIND_GET_CHECK_INTEGER(velocity, 2)
    // call the implementation
    BIND_WRAP_TRY(obj->velocity(strike, velocity))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger MidiTabToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Midi.Tab)", 10);
    return 1;
}
#undef BIND_CLASS_NAME


void bindMidi(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("Midi", 4);

    // static methods
    binding.addMethod("abc", 3, &Midiabc);

    // create class Midi.Abc
    binding.startClass("Abc", 3, MidiAbcObject);
    binding.addMethod("constructor", 11, &MidiAbcCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Abc>); // clone
    binding.addMethod("abc", 3, &MidiAbcabc);
    binding.addMethod("file", 4, &MidiAbcfile);
    binding.addMethod("sequence", 8, &MidiAbcsequence);
    binding.addMethod("_tostring", 9, &MidiAbcToString);
    binding.endClass();

    // create class Midi.BeatTracker
    binding.startClass("BeatTracker", 11, MidiBeatTrackerObject);
    binding.addMethod("constructor", 11, &MidiBeatTrackerCtor); // ctor
    binding.addMethod("_cloned", 7, &MidiBeatTrackerNoclone); // unclonable
    binding.addMethod("start", 5, &MidiBeatTrackerstart);
    binding.addMethod("stop", 4, &MidiBeatTrackerstop);
    binding.addMethod("_tostring", 9, &MidiBeatTrackerToString);
    binding.endClass();

    // create class Midi.ClockGenerator
    binding.startClass("ClockGenerator", 14, MidiClockGeneratorObject);
    binding.addMethod("constructor", 11, &MidiClockGeneratorCtor); // ctor
    binding.addMethod("_cloned", 7, &MidiClockGeneratorNoclone); // unclonable
    binding.addMethod("_get", 4, &MidiClockGeneratorGetters);
    binding.addMethod("_set", 4, &MidiClockGeneratorSetters);
    binding.addMethod("_tostring", 9, &MidiClockGeneratorToString);
    binding.endClass();

    // create class Midi.Connection
    binding.startClass("Connection", 10, MidiConnectionObject);
    binding.addMethod("constructor", 11, &MidiConnectionCtor); // ctor
    binding.addMethod("_cloned", 7, &MidiConnectionNoclone); // unclonable
    binding.addMethod("onControl", 9, &MidiConnectiononControl);
    binding.addMethod("onNoteOff", 9, &MidiConnectiononNoteOff);
    binding.addMethod("onNoteOn", 8, &MidiConnectiononNoteOn);
    binding.addMethod("onProgramChange", 15, &MidiConnectiononProgramChange);
    binding.addMethod("_tostring", 9, &MidiConnectionToString);
    binding.endClass();

    // create class Midi.Connector
    binding.startClass("Connector", 9, MidiConnectorObject);
    binding.addMethod("constructor", 11, &MidiConnectorCtor); // ctor
    binding.addMethod("_cloned", 7, &MidiConnectorNoclone); // unclonable
    binding.addMethod("clear", 5, &MidiConnectorclear);
    binding.addMethod("schedule", 8, &MidiConnectorschedule);
    binding.addMethod("_tostring", 9, &MidiConnectorToString);
    binding.endClass();

    // create class Midi.Copyright
    binding.startClass("Copyright", 9, MidiCopyrightObject);
    binding.addMethod("constructor", 11, &MidiCopyrightCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Copyright>); // clone
    binding.addMethod("_get", 4, &MidiCopyrightGetters);
    binding.addMethod("_set", 4, &MidiCopyrightSetters);
    binding.addMethod("_tostring", 9, &MidiCopyrightToString);
    binding.endClass();

    // create class Midi.CuePoint
    binding.startClass("CuePoint", 8, MidiCuePointObject);
    binding.addMethod("constructor", 11, &MidiCuePointCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<CuePoint>); // clone
    binding.addMethod("_get", 4, &MidiCuePointGetters);
    binding.addMethod("_set", 4, &MidiCuePointSetters);
    binding.addMethod("_tostring", 9, &MidiCuePointToString);
    binding.endClass();

    // create class Midi.MessageEvent
    binding.startClass("MessageEvent", 12, MidiMessageEventObject);
    binding.addMethod("constructor", 11, &MidiMessageEventCtor); // ctor
    binding.addMethod("_cloned", 7, &MidiMessageEventNoclone); // unclonable
    binding.addMethod("_get", 4, &MidiMessageEventGetters);
    binding.addMethod("_set", 4, &MidiMessageEventSetters);
    binding.addMethod("_tostring", 9, &MidiMessageEventToString);
    binding.endClass();

    // create class Midi.File
    binding.startClass("File", 4, MidiFileObject);
    binding.addMethod("constructor", 11, &MidiFileCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<File>); // clone
    binding.addMethod("_get", 4, &MidiFileGetters);
    binding.addMethod("_set", 4, &MidiFileSetters);
    binding.addMethod("save", 4, &MidiFilesave);
    binding.addMethod("tracks", 6, &MidiFiletracks);
    binding.addMethod("_tostring", 9, &MidiFileToString);
    binding.endClass();

    // create class Midi.Input
    binding.startClass("Input", 5, MidiInputObject);
    binding.addMethod("constructor", 11, &MidiInputCtor); // ctor
    binding.addMethod("_cloned", 7, &MidiInputNoclone); // unclonable
    binding.addMethod("_get", 4, &MidiInputGetters);
    binding.addMethod("_set", 4, &MidiInputSetters);
    binding.addMethod("onControl", 9, &MidiInputonControl);
    binding.addMethod("onNoteOff", 9, &MidiInputonNoteOff);
    binding.addMethod("onNoteOn", 8, &MidiInputonNoteOn);
    binding.addMethod("onProgramChange", 15, &MidiInputonProgramChange);
    binding.addMethod("_tostring", 9, &MidiInputToString);
    binding.endClass();

    // create class Midi.Instrument
    binding.startClass("Instrument", 10, MidiInstrumentObject);
    binding.addMethod("constructor", 11, &MidiInstrumentCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Instrument>); // clone
    binding.addMethod("_get", 4, &MidiInstrumentGetters);
    binding.addMethod("_set", 4, &MidiInstrumentSetters);
    binding.addMethod("_tostring", 9, &MidiInstrumentToString);
    binding.endClass();

    // create class Midi.Lyric
    binding.startClass("Lyric", 5, MidiLyricObject);
    binding.addMethod("constructor", 11, &MidiLyricCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Lyric>); // clone
    binding.addMethod("_get", 4, &MidiLyricGetters);
    binding.addMethod("_set", 4, &MidiLyricSetters);
    binding.addMethod("_tostring", 9, &MidiLyricToString);
    binding.endClass();

    // create class Midi.Marker
    binding.startClass("Marker", 6, MidiMarkerObject);
    binding.addMethod("constructor", 11, &MidiMarkerCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Marker>); // clone
    binding.addMethod("_get", 4, &MidiMarkerGetters);
    binding.addMethod("_set", 4, &MidiMarkerSetters);
    binding.addMethod("_tostring", 9, &MidiMarkerToString);
    binding.endClass();

    // create class Midi.Note
    binding.startClass("Note", 4, MidiNoteObject);
    binding.addMethod("constructor", 11, &MidiNoteCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Note>); // clone
    binding.addMethod("_get", 4, &MidiNoteGetters);
    binding.addMethod("_set", 4, &MidiNoteSetters);
    binding.addMethod("_tostring", 9, &MidiNoteToString);
    binding.endClass();

    // create class Midi.NoteEvent
    binding.startClass("NoteEvent", 9, MidiNoteEventObject);
    binding.addMethod("constructor", 11, &MidiNoteEventCtor); // ctor
    binding.addMethod("_cloned", 7, &MidiNoteEventNoclone); // unclonable
    binding.addMethod("_get", 4, &MidiNoteEventGetters);
    binding.addMethod("_set", 4, &MidiNoteEventSetters);
    binding.addMethod("_tostring", 9, &MidiNoteEventToString);
    binding.endClass();

    // create class Midi.NoteOn
    binding.startClass("NoteOn", 6, MidiNoteOnObject);
    binding.addMethod("constructor", 11, &MidiNoteOnCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<NoteOn>); // clone
    binding.addMethod("_get", 4, &MidiNoteOnGetters);
    binding.addMethod("_set", 4, &MidiNoteOnSetters);
    binding.addMethod("_tostring", 9, &MidiNoteOnToString);
    binding.endClass();

    // create class Midi.NoteOff
    binding.startClass("NoteOff", 7, MidiNoteOffObject);
    binding.addMethod("constructor", 11, &MidiNoteOffCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<NoteOff>); // clone
    binding.addMethod("_get", 4, &MidiNoteOffGetters);
    binding.addMethod("_set", 4, &MidiNoteOffSetters);
    binding.addMethod("_tostring", 9, &MidiNoteOffToString);
    binding.endClass();

    // create class Midi.KeySignature
    binding.startClass("KeySignature", 12, MidiKeySignatureObject);
    binding.addMethod("constructor", 11, &MidiKeySignatureCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<KeySignature>); // clone
    binding.addMethod("_get", 4, &MidiKeySignatureGetters);
    binding.addMethod("_set", 4, &MidiKeySignatureSetters);
    binding.addMethod("_tostring", 9, &MidiKeySignatureToString);
    binding.endClass();

    // create class Midi.Control
    binding.startClass("Control", 7, MidiControlObject);
    binding.addMethod("constructor", 11, &MidiControlCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Control>); // clone
    binding.addMethod("_get", 4, &MidiControlGetters);
    binding.addMethod("_set", 4, &MidiControlSetters);
    binding.addMethod("_tostring", 9, &MidiControlToString);
    binding.endClass();

    // create class Midi.MML
    binding.startClass("MML", 3, MidiMMLObject);
    binding.addMethod("constructor", 11, &MidiMMLCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<MML>); // clone
    binding.addMethod("sequence", 8, &MidiMMLsequence);
    binding.addMethod("_tostring", 9, &MidiMMLToString);
    binding.endClass();

    // create class Midi.Pattern
    binding.startClass("Pattern", 7, MidiPatternObject);
    binding.addMethod("constructor", 11, &MidiPatternCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Pattern>); // clone
    binding.addMethod("_get", 4, &MidiPatternGetters);
    binding.addMethod("_nexti", 6, &MidiPatternNexti);
    binding.addMethod("add", 3, &MidiPatternadd);
    binding.addMethod("filter", 6, &MidiPatternfilter);
    binding.addMethod("len", 3, &MidiPatternlen);
    binding.addMethod("print", 5, &MidiPatternprint);
    binding.addMethod("sequence", 8, &MidiPatternsequence);
    binding.addMethod("slice", 5, &MidiPatternslice);
    binding.addMethod("transpose", 9, &MidiPatterntranspose);
    binding.addMethod("_tostring", 9, &MidiPatternToString);
    binding.endClass();

    // create class Midi.Sequence
    binding.startClass("Sequence", 8, MidiSequenceObject);
    binding.addMethod("constructor", 11, &MidiSequenceCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Sequence>); // clone
    binding.addMethod("_get", 4, &MidiSequenceGetters);
    binding.addMethod("_nexti", 6, &MidiSequenceNexti);
    binding.addMethod("add", 3, &MidiSequenceadd);
    binding.addMethod("filter", 6, &MidiSequencefilter);
    binding.addMethod("len", 3, &MidiSequencelen);
    binding.addMethod("pattern", 7, &MidiSequencepattern);
    binding.addMethod("_tostring", 9, &MidiSequenceToString);
    binding.endClass();

    // create class Midi.TempoChange
    binding.startClass("TempoChange", 11, MidiTempoChangeObject);
    binding.addMethod("constructor", 11, &MidiTempoChangeCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<TempoChange>); // clone
    binding.addMethod("_get", 4, &MidiTempoChangeGetters);
    binding.addMethod("_set", 4, &MidiTempoChangeSetters);
    binding.addMethod("_tostring", 9, &MidiTempoChangeToString);
    binding.endClass();

    // create class Midi.Text
    binding.startClass("Text", 4, MidiTextObject);
    binding.addMethod("constructor", 11, &MidiTextCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Text>); // clone
    binding.addMethod("_get", 4, &MidiTextGetters);
    binding.addMethod("_set", 4, &MidiTextSetters);
    binding.addMethod("_tostring", 9, &MidiTextToString);
    binding.endClass();

    // create class Midi.TimeSignature
    binding.startClass("TimeSignature", 13, MidiTimeSignatureObject);
    binding.addMethod("constructor", 11, &MidiTimeSignatureCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<TimeSignature>); // clone
    binding.addMethod("_get", 4, &MidiTimeSignatureGetters);
    binding.addMethod("_set", 4, &MidiTimeSignatureSetters);
    binding.addMethod("_tostring", 9, &MidiTimeSignatureToString);
    binding.endClass();

    // create class Midi.TrackName
    binding.startClass("TrackName", 9, MidiTrackNameObject);
    binding.addMethod("constructor", 11, &MidiTrackNameCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<TrackName>); // clone
    binding.addMethod("_get", 4, &MidiTrackNameGetters);
    binding.addMethod("_set", 4, &MidiTrackNameSetters);
    binding.addMethod("_tostring", 9, &MidiTrackNameToString);
    binding.endClass();

    // create class Midi.Output
    binding.startClass("Output", 6, MidiOutputObject);
    binding.addMethod("constructor", 11, &MidiOutputCtor); // ctor
    binding.addMethod("_cloned", 7, &MidiOutputNoclone); // unclonable
    binding.addMethod("_get", 4, &MidiOutputGetters);
    binding.addMethod("_set", 4, &MidiOutputSetters);
    binding.addMethod("clear", 5, &MidiOutputclear);
    binding.addMethod("remove", 6, &MidiOutputremove);
    binding.addMethod("schedule", 8, &MidiOutputschedule);
    binding.addMethod("send", 4, &MidiOutputsend);
    binding.addMethod("_tostring", 9, &MidiOutputToString);
    binding.endClass();

    // create class Midi.PitchBend
    binding.startClass("PitchBend", 9, MidiPitchBendObject);
    binding.addMethod("constructor", 11, &MidiPitchBendCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<PitchBend>); // clone
    binding.addMethod("_get", 4, &MidiPitchBendGetters);
    binding.addMethod("_set", 4, &MidiPitchBendSetters);
    binding.addMethod("_tostring", 9, &MidiPitchBendToString);
    binding.endClass();

    // create class Midi.ProgramChange
    binding.startClass("ProgramChange", 13, MidiProgramChangeObject);
    binding.addMethod("constructor", 11, &MidiProgramChangeCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<ProgramChange>); // clone
    binding.addMethod("_get", 4, &MidiProgramChangeGetters);
    binding.addMethod("_set", 4, &MidiProgramChangeSetters);
    binding.addMethod("_tostring", 9, &MidiProgramChangeToString);
    binding.endClass();

    // create class Midi.Tab
    binding.startClass("Tab", 3, MidiTabObject);
    binding.addMethod("constructor", 11, &MidiTabCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Tab>); // clone
    binding.addMethod("_get", 4, &MidiTabGetters);
    binding.addMethod("_set", 4, &MidiTabSetters);
    binding.addMethod("define", 6, &MidiTabdefine);
    binding.addMethod("sequence", 8, &MidiTabsequence);
    binding.addMethod("string", 6, &MidiTabstring);
    binding.addMethod("velocity", 8, &MidiTabvelocity);
    binding.addMethod("_tostring", 9, &MidiTabToString);
    binding.endClass();

    binding.endTable(); // end package table
}
}}
