/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "bindosc.h"
#include "bindinterface.h"
#include "bindutil.h"

#include "oscinput.h"
#include "oscmessage.h"
#include "oscoutput.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

using namespace osc;

namespace binding {

// object references to types in this package
HSQOBJECT OscInputObject;
HSQOBJECT OscMessageObject;
HSQOBJECT OscOutputObject;

//
// Osc.Input class
//
#define BIND_CLASS_NAME "Input"
#define BIND_METHOD_NAME "constructor"
SQInteger OscInputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Input *obj;
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj = osc::InputFactory::instance().newOscInput())
    } else {
        HSQOBJECT arg1;
        sq_getstackobj(vm, 2, &arg1);
        if(arg1._type == OT_INTEGER) {
            BIND_NARGS_CHECK_MAX(2) // check max parameter count
            // get parameter 1 "port" as integer
            BIND_GET_INTEGER(port, 1)
            if(numargs == 2) {
                // call the implementation
                BIND_WRAP_TRY(obj = osc::InputFactory::instance().newOscInput(port))
            } else {
                HSQOBJECT arg2;
                sq_getstackobj(vm, 3, &arg2);
                if(arg2._type == OT_INTEGER) {
                    // get parameter 2 "protocol" as integer
                    BIND_GET_INTEGER(protocol, 2)
                    // call the implementation
                    BIND_WRAP_TRY(obj = osc::InputFactory::instance().newOscInput(port, protocol))
                }
                else if(arg2._type == OT_STRING) {
                    BIND_NARGS_CHECK_MIN(2) // check min parameter count
                    // get parameter 2 "address" as string
                    BIND_GET_STRING(address, 2)
                    // call the implementation
                    BIND_WRAP_TRY(obj = osc::InputFactory::instance().newOscInput(port, address))
                }
                else {
                    return sq_throwerror(vm, "argument 2 is not of type {integer, string}");
                }
            }
        }
        else if(arg1._type == OT_STRING) {
            BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
            // get parameter 1 "path" as string
            BIND_GET_STRING(path, 1)
            // call the implementation
            BIND_WRAP_TRY(obj = osc::InputFactory::instance().newOscInput(path))
        }
        else {
            return sq_throwerror(vm, "argument 1 is not of type {integer, string}");
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger OscInputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Osc.Input");
}
//
// Osc.Input property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger OscInputGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(osc::Input)
    // dispatch appropriate getter
    if(!strcmp(key, "port")) {
        SQInteger ret = obj->port();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Osc.Input property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger OscInputSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(osc::Input)
    // dispatch appropriate setter
    if(!strcmp(key, "port")) {
        return sq_throwerror(vm, "property \"port\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Osc.Input onReceive
//
#define BIND_METHOD_NAME "onReceive"
SQInteger OscInputonReceive(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Input)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "handler" as function
    BIND_GET_CHECK_FUNCTION(handler, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->onReceive(handler))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger OscInputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Osc.Input)", 11);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Osc.Message class
//
#define BIND_CLASS_NAME "Message"
#define BIND_METHOD_NAME "constructor"
SQInteger OscMessageCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Message *obj;
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "path" as string
    BIND_GET_CHECK_STRING(path, 1)
    // call the implementation
    BIND_WRAP_TRY(obj = new Message(path))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Message>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// Osc.Message property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger OscMessageGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(osc::Message)
    // dispatch appropriate getter
    if(!strcmp(key, "path")) {
        std::string ret = obj->path();
        BIND_PUSH_STRING(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Osc.Message property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger OscMessageSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(osc::Message)
    // dispatch appropriate setter
    if(!strcmp(key, "path")) {
        return sq_throwerror(vm, "property \"path\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// Osc.Message add
//
#define BIND_METHOD_NAME "add"
SQInteger OscMessageadd(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Message)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    HSQOBJECT arg1;
    sq_getstackobj(vm, 2, &arg1);
    if(arg1._type == OT_INTEGER) {
        // get parameter 1 "value" as integer
        BIND_GET_INTEGER(value, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->addInteger(value))
    }
    else if(BIND_CHECK_FLOAT(1)) {
        // get parameter 1 "value" as float
        BIND_GET_FLOAT(value, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->addFloat(value))
    }
    else if(arg1._type == OT_STRING) {
        // get parameter 1 "value" as string
        BIND_GET_STRING(value, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->addString(value))
    }
    else if(arg1._type == OT_BOOL) {
        // get parameter 1 "value" as bool
        BIND_GET_BOOL(value, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->addBoolean(value))
    }
    else if(arg1._type == OT_NULL) {
        // get parameter 1 "value" as null
        BIND_GET_NULL(value, 1)
        // call the implementation
        BIND_WRAP_TRY(obj->addNull(value))
    }
    else {
        return sq_throwerror(vm, "argument 1 is not of type {integer, float, string, bool, null}");
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Osc.Message arg
//
#define BIND_METHOD_NAME "arg"
SQInteger OscMessagearg(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Message)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "index" as integer
    BIND_GET_CHECK_INTEGER(index, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->arg(vm, index))
    return 1;
}
#undef BIND_METHOD_NAME

//
// Osc.Message args
//
#define BIND_METHOD_NAME "args"
SQInteger OscMessageargs(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Message)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    SQInteger ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->parameterCount())
    // push return value
    BIND_PUSH_INTEGER(ret);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger OscMessageToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Osc.Message)", 13);
    return 1;
}
#undef BIND_CLASS_NAME

//
// Osc.Output class
//
#define BIND_CLASS_NAME "Output"
#define BIND_METHOD_NAME "constructor"
SQInteger OscOutputCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Output *obj;
    BIND_NARGS_CHECK_EXACT(2) // check exact parameter count
    // get parameter 1 "host" as string
    BIND_GET_CHECK_STRING(host, 1)
    // get parameter 2 "port" as integer
    BIND_GET_CHECK_INTEGER(port, 2)
    // call the implementation
    BIND_WRAP_TRY(obj = osc::OutputFactory::instance().newOscOutput(host, port))
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger OscOutputNoclone(HSQUIRRELVM vm)
{
    return sq_throwerror(vm, "cannot clone objects of type Osc.Output");
}
//
// Osc.Output schedule
//
#define BIND_METHOD_NAME "schedule"
SQInteger OscOutputschedule(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Output)
    BIND_NARGS_CHECK_MIN(2) // check min parameter count
    BIND_NARGS_CHECK_MAX(3) // check max parameter count
    // get parameter 1 "message" as Osc.Message
    BIND_GET_CHECK_OBJECT(message, 1, Osc, Message, osc::Message)
    // get parameter 2 "bar" as float
    BIND_GET_CHECK_FLOAT(bar, 2)
    if(numargs == 3) {
        // call the implementation
        BIND_WRAP_TRY(obj->schedule(*message, bar))
    } else {
        // get parameter 3 "clock" as Time.Clock
        BIND_GET_CHECK_INTERFACE(clock, 3, Time, Clock, time::Clock)
        // call the implementation
        BIND_WRAP_TRY(obj->schedule(*message, bar, *clock))
    }
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// Osc.Output send
//
#define BIND_METHOD_NAME "send"
SQInteger OscOutputsend(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Output)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "message" as Osc.Message
    BIND_GET_CHECK_OBJECT(message, 1, Osc, Message, osc::Message)
    // call the implementation
    BIND_WRAP_TRY(obj->send(*message))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

SQInteger OscOutputToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(Osc.Output)", 12);
    return 1;
}
#undef BIND_CLASS_NAME


void bindOsc(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("Osc", 3);

    // create class Osc.Input
    binding.startClass("Input", 5, OscInputObject);
    binding.addMethod("constructor", 11, &OscInputCtor); // ctor
    binding.addMethod("_cloned", 7, &OscInputNoclone); // unclonable
    binding.addMethod("_get", 4, &OscInputGetters);
    binding.addMethod("_set", 4, &OscInputSetters);
    binding.addMethod("onReceive", 9, &OscInputonReceive);
    binding.addMethod("_tostring", 9, &OscInputToString);
    binding.endClass();

    // create class Osc.Message
    binding.startClass("Message", 7, OscMessageObject);
    binding.addMethod("constructor", 11, &OscMessageCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Message>); // clone
    binding.addMethod("_get", 4, &OscMessageGetters);
    binding.addMethod("_set", 4, &OscMessageSetters);
    binding.addMethod("add", 3, &OscMessageadd);
    binding.addMethod("arg", 3, &OscMessagearg);
    binding.addMethod("args", 4, &OscMessageargs);
    binding.addMethod("_tostring", 9, &OscMessageToString);
    binding.endClass();

    // create class Osc.Output
    binding.startClass("Output", 6, OscOutputObject);
    binding.addMethod("constructor", 11, &OscOutputCtor); // ctor
    binding.addMethod("_cloned", 7, &OscOutputNoclone); // unclonable
    binding.addMethod("schedule", 8, &OscOutputschedule);
    binding.addMethod("send", 4, &OscOutputsend);
    binding.addMethod("_tostring", 9, &OscOutputToString);
    binding.endClass();

    binding.endTable(); // end package table
}
}}
