/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "bindsystem.h"
#include "bindinterface.h"
#include "bindutil.h"

#include "memory.h"
#include "systemtime.h"
#include <stdexcept>
#include <cstring>

namespace bipscript {

using namespace system;

namespace binding {

// object references to types in this package
HSQOBJECT SystemMemoryObject;
HSQOBJECT SystemTimeObject;

//
// System.Memory class
//
#define BIND_CLASS_NAME "Memory"
#define BIND_METHOD_NAME "constructor"
SQInteger SystemMemoryCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Memory *obj;
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // call the implementation
    BIND_WRAP_TRY(obj = new Memory())
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Memory>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// System.Memory property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger SystemMemoryGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(system::Memory)
    // dispatch appropriate getter
    if(!strcmp(key, "total")) {
        SQInteger ret = obj->total();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "free")) {
        SQInteger ret = obj->free();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "resident")) {
        SQInteger ret = obj->resident();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "used")) {
        SQInteger ret = obj->used();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// System.Memory property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger SystemMemorySetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(system::Memory)
    // dispatch appropriate setter
    if(!strcmp(key, "total")) {
        return sq_throwerror(vm, "property \"total\" is read-only");
    }
    if(!strcmp(key, "free")) {
        return sq_throwerror(vm, "property \"free\" is read-only");
    }
    if(!strcmp(key, "resident")) {
        return sq_throwerror(vm, "property \"resident\" is read-only");
    }
    if(!strcmp(key, "used")) {
        return sq_throwerror(vm, "property \"used\" is read-only");
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

SQInteger SystemMemoryToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(System.Memory)", 15);
    return 1;
}
#undef BIND_CLASS_NAME

//
// System.Time class
//
#define BIND_CLASS_NAME "Time"
#define BIND_METHOD_NAME "constructor"
SQInteger SystemTimeCtor(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    Time *obj;
    if(numargs == 1) {
        // call the implementation
        BIND_WRAP_TRY(obj = new Time())
    } else {
        HSQOBJECT arg1;
        sq_getstackobj(vm, 2, &arg1);
        if(arg1._type == OT_STRING) {
            BIND_NARGS_CHECK_MAX(1) // check max parameter count
            // get parameter 1 "iso" as string
            BIND_GET_STRING(iso, 1)
            // call the implementation
            BIND_WRAP_TRY(obj = new Time(iso))
        }
        else if(arg1._type == OT_INTEGER) {
            BIND_NARGS_CHECK_MIN(3) // check min parameter count
            BIND_NARGS_CHECK_MAX(6) // check max parameter count
            // get parameter 1 "year" as integer
            BIND_GET_INTEGER(year, 1)
            // get parameter 2 "month" as integer
            BIND_GET_CHECK_INTEGER(month, 2)
            // get parameter 3 "day" as integer
            BIND_GET_CHECK_INTEGER(day, 3)
            if(numargs == 4) {
                // call the implementation
                BIND_WRAP_TRY(obj = new Time(year, month, day))
            } else {
                // get parameter 4 "hour" as integer
                BIND_GET_CHECK_INTEGER(hour, 4)
                if(numargs == 5) {
                    // call the implementation
                    BIND_WRAP_TRY(obj = new Time(year, month, day, hour))
                } else {
                    // get parameter 5 "minute" as integer
                    BIND_GET_CHECK_INTEGER(minute, 5)
                    if(numargs == 6) {
                        // call the implementation
                        BIND_WRAP_TRY(obj = new Time(year, month, day, hour, minute))
                    } else {
                        // get parameter 6 "second" as integer
                        BIND_GET_CHECK_INTEGER(second, 6)
                        // call the implementation
                        BIND_WRAP_TRY(obj = new Time(year, month, day, hour, minute, second))
                    }
                }
            }
        }
        else {
            return sq_throwerror(vm, "argument 1 is not of type {string, integer}");
        }
    }
    // return pointer to new object
    sq_setinstanceup(vm, 1, (SQUserPointer*)obj);
    sq_setreleasehook(vm, 1, BindUtil::Delete<Time>);
    return 1;
}
#undef BIND_METHOD_NAME

//
// System.Time property getters
//
#define BIND_METHOD_NAME "getter"
SQInteger SystemTimeGetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(system::Time)
    // dispatch appropriate getter
    if(!strcmp(key, "year")) {
        SQInteger ret = obj->year();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "month")) {
        SQInteger ret = obj->month();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "day")) {
        SQInteger ret = obj->day();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "weekday")) {
        SQInteger ret = obj->weekday();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "hour")) {
        SQInteger ret = obj->hour();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "minute")) {
        SQInteger ret = obj->minute();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    if(!strcmp(key, "second")) {
        SQInteger ret = obj->second();
        BIND_PUSH_INTEGER(ret);
        return 1;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// System.Time property setters
//
#define BIND_METHOD_NAME "setter"
SQInteger SystemTimeSetters(HSQUIRRELVM vm)
{
    SQObjectType argtype = sq_gettype(vm, 2);
    if(argtype == OT_STRING) {
    // get property name
    const SQChar* key;
    sq_getstring(vm, 2, &key);
    BIND_GET_THIS_PROPERTY(system::Time)
    // dispatch appropriate setter
    if(!strcmp(key, "year")) {
        // get parameter 2 "year" as integer
        BIND_GET_CHECK_INTEGER(year, 2)
        BIND_WRAP_TRY(obj->year(year))
        return 0;
    }
    if(!strcmp(key, "month")) {
        // get parameter 2 "month" as integer
        BIND_GET_CHECK_INTEGER(month, 2)
        BIND_WRAP_TRY(obj->month(month))
        return 0;
    }
    if(!strcmp(key, "day")) {
        // get parameter 2 "day" as integer
        BIND_GET_CHECK_INTEGER(day, 2)
        BIND_WRAP_TRY(obj->day(day))
        return 0;
    }
    if(!strcmp(key, "weekday")) {
        return sq_throwerror(vm, "property \"weekday\" is read-only");
    }
    if(!strcmp(key, "hour")) {
        // get parameter 2 "hour" as integer
        BIND_GET_CHECK_INTEGER(hour, 2)
        BIND_WRAP_TRY(obj->hour(hour))
        return 0;
    }
    if(!strcmp(key, "minute")) {
        // get parameter 2 "minute" as integer
        BIND_GET_CHECK_INTEGER(minute, 2)
        BIND_WRAP_TRY(obj->minute(minute))
        return 0;
    }
    if(!strcmp(key, "second")) {
        // get parameter 2 "second" as integer
        BIND_GET_CHECK_INTEGER(second, 2)
        BIND_WRAP_TRY(obj->second(second))
        return 0;
    }
    }
    sq_pushnull(vm);
    return sq_throwobject(vm);
}
#undef BIND_METHOD_NAME

//
// System.Time date
//
#define BIND_METHOD_NAME "date"
SQInteger SystemTimedate(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Time)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    std::string ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->date())
    // push return value
    BIND_PUSH_STRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// System.Time dayname
//
#define BIND_METHOD_NAME "dayname"
SQInteger SystemTimedayname(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Time)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    std::string ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->dayname())
    // push return value
    BIND_PUSH_STRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// System.Time format
//
#define BIND_METHOD_NAME "format"
SQInteger SystemTimeformat(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Time)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "specifiers" as string
    BIND_GET_CHECK_STRING(specifiers, 1)
    // return value
    std::string ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->format(specifiers))
    // push return value
    BIND_PUSH_STRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// System.Time monthname
//
#define BIND_METHOD_NAME "monthname"
SQInteger SystemTimemonthname(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Time)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    std::string ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->monthname())
    // push return value
    BIND_PUSH_STRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

//
// System.Time set
//
#define BIND_METHOD_NAME "set"
SQInteger SystemTimeset(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Time)
    BIND_NARGS_CHECK_EXACT(1) // check exact parameter count
    // get parameter 1 "iso" as string
    BIND_GET_CHECK_STRING(iso, 1)
    // call the implementation
    BIND_WRAP_TRY(obj->set(iso))
    // void method, returns no value
    return 0;
}
#undef BIND_METHOD_NAME

//
// System.Time time
//
#define BIND_METHOD_NAME "time"
SQInteger SystemTimetime(HSQUIRRELVM vm)
{
    BIND_GET_NARGS()
    BIND_GET_THIS_INSTANCE(Time)
    BIND_NARGS_CHECK_EXACT(0) // check exact parameter count
    // return value
    std::string ret;
    // call the implementation
    BIND_WRAP_TRY(ret = obj->time())
    // push return value
    BIND_PUSH_STRING(ret);
    return 1;
}
#undef BIND_METHOD_NAME

SQInteger SystemTimeToString(HSQUIRRELVM vm)
{
    sq_pushstring(vm, "(System.Time)", 13);
    return 1;
}
#undef BIND_CLASS_NAME


void bindSystem(HSQUIRRELVM vm)
{
    BindUtil binding(vm);
    binding.startTable("System", 6);

    // create class System.Memory
    binding.startClass("Memory", 6, SystemMemoryObject);
    binding.addMethod("constructor", 11, &SystemMemoryCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Memory>); // clone
    binding.addMethod("_get", 4, &SystemMemoryGetters);
    binding.addMethod("_set", 4, &SystemMemorySetters);
    binding.addMethod("_tostring", 9, &SystemMemoryToString);
    binding.endClass();

    // create class System.Time
    binding.startClass("Time", 4, SystemTimeObject);
    binding.addMethod("constructor", 11, &SystemTimeCtor); // ctor
    binding.addMethod("_cloned", 7, BindUtil::Clone<Time>); // clone
    binding.addMethod("_get", 4, &SystemTimeGetters);
    binding.addMethod("_set", 4, &SystemTimeSetters);
    binding.addMethod("date", 4, &SystemTimedate);
    binding.addMethod("dayname", 7, &SystemTimedayname);
    binding.addMethod("format", 6, &SystemTimeformat);
    binding.addMethod("monthname", 9, &SystemTimemonthname);
    binding.addMethod("set", 3, &SystemTimeset);
    binding.addMethod("time", 4, &SystemTimetime);
    binding.addMethod("_tostring", 9, &SystemTimeToString);
    binding.endClass();

    binding.endTable(); // end package table
}
}}
