/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef BIPXTAPI_H
#define BIPXTAPI_H

#include <stdint.h>

#define BXT_FAILED(result) (result < 0)

extern "C" {

typedef uint32_t BXTBoolean;
#ifdef _LP64
typedef long long BXTInteger;
#else
typedef int BXTInteger;
#endif
typedef float BXTFloat;

typedef int32_t BXTResult;
typedef int32_t BXTIndex;
typedef BXTResult (*BXTMethod)(BXTIndex);

typedef void *BXTClass;
typedef void *BXTFunction;
typedef void *BXTFunctionCall;
typedef void *BXTInstance;
typedef void *BXTProcessThread;
typedef void *BXTTimeline;

typedef BXTResult (*BXTAudioProcess)(BXTInstance, BXTIndex);

// init + shutdowns
BXTResult bipxt_ext_init(uint32_t version);
void bipxt_ext_init_pack(const char *name);
void bipxt_ext_shutdown();

// process thread
typedef int (*BXTProcess)(void *);
BXTProcessThread bipxt_create_thread(BXTProcess process, BXTProcess restart, void *arg);

// scheduling via timeline
BXTTimeline bipxt_timeline_create();
void bipxt_timeline_schedule(BXTTimeline, void *event, float measure); // TODO: non-default clock
void *bipxt_timeline_event_first(BXTTimeline);
void *bipxt_timeline_event_next(BXTTimeline);
int bipxt_timeline_active(BXTTimeline);

// binding
BXTClass bipxt_bind_class(const char *name, uint32_t namelen, BXTMethod ctor);
void bipxt_bind_method(const char *name, uint32_t namelen, BXTMethod func);
void bipxt_bind_class_end();

void bipxt_bind_enumeration(const char *name);
void bipxt_bind_enumerator(const char *name, int32_t value);
void bipxt_bind_enumeration_end();

BXTResult bipxt_get_boolean(BXTIndex index, BXTBoolean *boolp);
void bipxt_put_boolean(BXTBoolean value);
BXTResult bipxt_get_integer(BXTIndex index, BXTInteger *intp);
void bipxt_put_integer(BXTInteger value);
BXTResult bipxt_get_float(BXTIndex index, BXTFloat *floatp);
void bipxt_put_float(BXTFloat value);
BXTResult bipxt_get_string(BXTIndex index, const char **strp);
void bipxt_put_string(const char *str, uint32_t strlen);

BXTResult bipxt_get_object(BXTIndex index, void **ptr, BXTClass type);
void bipxt_put_object(void *obj, void *type);
BXTResult bipxt_get_this(void **ptr);
void bipxt_put_this(const void *obj, uint32_t rdelete);
BXTResult bipxt_get_instance(BXTIndex index, BXTInstance *ptr);
BXTResult bipxt_get_function(BXTIndex index, BXTFunction *ptr);

// method calls
BXTFunctionCall bipxt_create_call(BXTFunction function);
void bipxt_call_add_integer(BXTFunctionCall call, BXTInteger value);
BXTResult bipxt_execute_call(BXTFunctionCall call, BXTInstance context);
BXTResult bipxt_execute(BXTFunction function, BXTInstance context);

void bipxt_add_reference(BXTInstance object);
void bipxt_event_handler(BXTMethod handler);

// audio processor
void bipxt_audio_processor(const char *name, BXTIndex inputs, BXTIndex outputs, void *obj,
                           BXTAudioProcess func);
BXTFloat *bipxt_audio_input_buffer(void *obj, BXTIndex index);
BXTFloat *bipxt_audio_output_buffer(void *obj, BXTIndex index);

// error handling
BXTResult bipxt_error(const char *mesg);
BXTResult bipxt_not_found();
}

#endif // BIPXTAPI_H
